<?xml version="1.0"?>

<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:srgs="http://www.w3.org/2001/06/grammar"
exclude-result-prefixes="srgs"
>
 

  <!--
    SRGS 1.0 Stylesheet to convert grammars from XML to ABNF Form (20030728)

    Copyright 1998-2003 W3C (MIT, ERCIM, Keio), All Rights Reserved. 

    Permission to use, copy, modify and distribute the SRGS XML to 
    ABNF stylesheet and its accompanying documentation
    for any purpose and without fee is hereby granted in
    perpetuity, provided that the above copyright notice and this
    paragraph appear in all copies.  The copyright holders make no
    representation about the suitability of the schema for any purpose. 
    It is provided "as is" without expressed or implied warranty.
    
    History: 05 2004 - Added tag attribute handling, and whitespace normalization.
                                               -Jia Pu and Michael Pust, LumenVox
    History: 11 2006 - Added empty item element handling, <item></item> -> NULL
											   -Chung Lai, LumenVox
    History: 10 2003 - Added quotes around DTMF * and # tokens
											   -Nigel Quinnin, LumenVox
    History: 01 2010 - Reformatted ABNF output, added weights to $NULL rules, 
                       removed unnecessary double $NULL rules, fixed header to
                       remove invalid xsl: scope from exclude-result-prefixes
											   -Nigel Quinnin, LumenVox
-->



<xsl:output method="text"  omit-xml-declaration="yes" standalone="no" doctype-system="grammar.dtd"/>

  <xsl:template name="addweight">
    <xsl:if test="string(@weight)!=''"> /<xsl:value-of select="@weight"/>/ </xsl:if>
  </xsl:template>

<xsl:template name="addnull">
    <xsl:if test="normalize-space(.)=''">$NULL</xsl:if>
  </xsl:template>

<xsl:template name="addlang">
    <xsl:if test="string(@xml:lang)!=''">
      !<xsl:value-of select="@xml:lang"/>
    </xsl:if>
  </xsl:template>

<xsl:template name="addtag">
    <xsl:if test="string(@tag)!=''">
      :"<xsl:value-of select="@tag"/>"
    </xsl:if>
  </xsl:template>

<xsl:template name="oparen"><xsl:if test="string(@xml:lang)!='' or string(@weight)!='' or string(@tag)!='' or string(@repeat-prob)!=''">(</xsl:if></xsl:template>

  <xsl:template name="cparen">
    <xsl:if test="string(@xml:lang)!='' or string(@weight)!='' or string(@tag)!='' or string(@repeat-prob)!=''">)</xsl:if>
  </xsl:template>


  <xsl:template name="addexamples">
    <xsl:if test="count(srgs:example)>0">
      /** <xsl:for-each select="srgs:example">
        * @example <xsl:value-of select="text()"/>
      </xsl:for-each>
      */
    </xsl:if>
  </xsl:template>

<xsl:template name="addrepeat">
    <xsl:if test="string(@repeat)!=''">
      <xsl:choose>
        <xsl:when test="string(@repeat-prob)!=''">
          <xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@repeat"/> /<xsl:value-of select="@repeat-prob"/>/<xsl:text disable-output-escaping="yes">&gt;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text disable-output-escaping="yes">&lt;</xsl:text>
          <xsl:value-of select="@repeat"/>
          <xsl:text disable-output-escaping="yes">&gt;</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>

<xsl:template match="srgs:grammar">#ABNF <xsl:value-of select="@version"/>;

/////////////////////////////////////////////////////////

<xsl:if test="string(@xml:base)!=''">base <xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@xml:base"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>;
    </xsl:if>
<xsl:if test="string(@xml:lang)!=''">language <xsl:value-of select="@xml:lang"/>;
</xsl:if><xsl:if test="string(@mode)!=''">mode <xsl:value-of select="@mode"/>;
</xsl:if><xsl:if test="string(@tag-format)!=''">tag-format <xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@tag-format"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>;
</xsl:if><xsl:apply-templates select="srgs:lexicon"/>
<xsl:apply-templates select="srgs:meta"/><xsl:if test="string(@root)!=''">root $<xsl:value-of disable-output-escaping="yes" select="@root"/>;
</xsl:if>
  <xsl:apply-templates select="srgs:tag" mode="header"/>
/////////////////////////////////////////////////////////

  <xsl:apply-templates select="srgs:rule"/>

  </xsl:template>

  <xsl:template match="srgs:tag" mode="header">
    {!{<xsl:value-of select="."/>}!};</xsl:template>

  <xsl:template match="srgs:tag">
    {!{<xsl:value-of select="."/>}!}
    </xsl:template>

  <xsl:template match="srgs:lexicon">
    <xsl:choose>
      <xsl:when test="string(@type)!=''">
        lexicon <xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@uri"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>~<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@type"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>;
      </xsl:when>
      <xsl:otherwise>
        lexicon <xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@uri"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>;
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Parametered meta data processing template -->
  <xsl:template name="metadata">
    <xsl:param name="tagname"/>
    <xsl:param name="metaname"/>
    <xsl:param name="metacontent"/>
    <xsl:param name="singlequote">'</xsl:param>
    <xsl:choose>
      <xsl:when test="contains($metacontent,$singlequote)">
        <xsl:value-of select="$tagname"/> "<xsl:value-of select="$metaname"/>" is "<xsl:value-of  disable-output-escaping="yes" select="$metacontent"/>";
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$tagname"/> '<xsl:value-of select="$metaname"/>' is '<xsl:value-of  disable-output-escaping="yes" select="$metacontent"/>';
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="srgs:meta">
    <xsl:if test="string(@content)!=''">
      <xsl:choose>
        <xsl:when test="string(@http-equiv)!=''">
          <xsl:call-template name="metadata">
            <xsl:with-param name="tagname">http-equiv</xsl:with-param>
            <xsl:with-param name="metaname">
              <xsl:value-of select="@name"/>
            </xsl:with-param>
            <xsl:with-param name="metacontent">
              <xsl:value-of select="@content"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:when>
        <xsl:when test="string(@name)!=''">
          <xsl:call-template name="metadata">
            <xsl:with-param name="tagname">meta</xsl:with-param>
            <xsl:with-param name="metaname">
              <xsl:value-of select="@name"/>
            </xsl:with-param>
            <xsl:with-param name="metacontent">
              <xsl:value-of select="@content"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>
    </xsl:if>
  </xsl:template>

  <!-- Axl: This template used to add double quotes around a string if it contained a comma or apostrophe if it didn't already contain double quotes. However, we don't need this anymore because we are disallowing single quotes in favor of apostrophes and quoting the entire string as a single token was causing problems
<xsl:template match="text()" mode="add-double-quotes">
<xsl:choose>
  <xsl:when test="normalize-space(.)!='' and contains(., '&quot;')"><xsl:value-of disable-output-escaping="yes" select="normalize-space(.)"/></xsl:when>
  <xsl:when test='normalize-space(.)!="" and (contains(., "&apos;") or contains(.,","))'>"<xsl:value-of disable-output-escaping="yes" select="normalize-space(.)"/>"</xsl:when>
  <xsl:otherwise><xsl:value-of disable-output-escaping="yes" select="normalize-space(.)"/></xsl:otherwise>
</xsl:choose>
</xsl:template>
-->
  <!-- NQ: This template is used to add double quotes around a single asterisk (*) or pound (#) that are not already surrounded by quotes -->
  <xsl:template match="text()" mode="add-double-quotes">
	<xsl:call-template name="recursive-add-double-quotes">
			<xsl:with-param name="text" select="normalize-space(.)"/>
	</xsl:call-template>
  </xsl:template>

  <xsl:template name="recursive-add-double-quotes">
  	<xsl:param name="text"/>
  	<xsl:choose>
		<!-- single quotes are handled as apostrophes by ABNF parser
  		<xsl:when test='contains($text,"&apos;")'>
  			<xsl:call-template name="recursive-add-double-quotes">
  				<xsl:with-param name="text" select='substring-before(substring-before($text,"&apos;")," ")'/>
  			</xsl:call-template>
 			<xsl:value-of select='substring-after(substring-before($text,"&apos;")," ")'/>
			"&apos;"
			<xsl:value-of select='substring-before(substring-after($text,"&apos;")," ")'/>
  			<xsl:call-template name="recursive-add-double-quotes">
  				<xsl:with-param name="text" select='substring-before(substring-after($text,"&apos;")," ")'/>
  			</xsl:call-template>
		</xsl:when>
		-->
  		<xsl:when test="contains($text,',')">
  			<xsl:call-template name="recursive-add-double-quotes">
  				<xsl:with-param name="text" select="substring-before($text,',')"/>
  			</xsl:call-template>
  			","
  			<xsl:call-template name="recursive-add-double-quotes">
  				<xsl:with-param name="text" select="substring-after($text,',')"/>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="contains($text,'#')">
  			<xsl:call-template name="recursive-add-double-quotes">
  				<xsl:with-param name="text" select="substring-before($text,'#')"/>
  			</xsl:call-template>
  			"#"
  			<xsl:call-template name="recursive-add-double-quotes">
  				<xsl:with-param name="text" select="substring-after($text,'#')"/>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:when test="contains($text,'*')">
  			<xsl:call-template name="recursive-add-double-quotes">
	  			<xsl:with-param name="text" select="substring-before($text,'*')"/>
  			</xsl:call-template>
  			"*"
  			<xsl:call-template name="recursive-add-double-quotes">
  				<xsl:with-param name="text" select="substring-after($text,'*')"/>
  			</xsl:call-template>
  		</xsl:when>
  		<xsl:otherwise>
			<xsl:value-of disable-output-escaping="yes" select="$text"/>
		</xsl:otherwise>
  	</xsl:choose>
</xsl:template>

<!-- SM: Handles double-quoted text. Internally invokes templates to surround '*', '#', ',' characters with double-quotes as needed -->
<xsl:template name="recursive-handle-double-quotes">
	<xsl:param name="text"/>
	<xsl:choose>
		<xsl:when test="contains($text,'&quot;')">
			<xsl:call-template name="recursive-add-double-quotes">
				<xsl:with-param name="text" select="substring-before($text,'&quot;')"/>
			</xsl:call-template>
			<xsl:variable name="text-after-first-quote" select="substring-after($text,'&quot;')" />
			<xsl:choose>
				<xsl:when test="contains($text-after-first-quote,'&quot;')">
					&quot;<xsl:value-of disable-output-escaping="yes" select="substring-before($text-after-first-quote,'&quot;')" />&quot;
					<xsl:call-template name="recursive-handle-double-quotes">
						<xsl:with-param name="text" select="substring-after($text-after-first-quote,'&quot;')"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="recursive-add-double-quotes">
						&quot;<xsl:with-param name="text" select="$text-after-first-quote"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="recursive-add-double-quotes">
				<xsl:with-param name="text" select="$text"/>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

  
<xsl:template match="srgs:rule">
  <xsl:call-template name="addexamples"/>
  <xsl:if test="@scope">
    <xsl:value-of select="@scope"/>
    <xsl:text> </xsl:text>
  </xsl:if>
$<xsl:value-of disable-output-escaping="yes" select="@id"/> = <xsl:apply-templates />;
</xsl:template>

  <xsl:template match="srgs:example"/>

  <xsl:template match="srgs:token">
    "<xsl:value-of disable-output-escaping="yes" select="text()"/>"<xsl:call-template name="addlang"/><xsl:call-template name="addtag"/>
  </xsl:template>

  <xsl:template match="text()">
    <xsl:if test="not(comment() |srgs:tag/text() | srgs:example/text())">
		<xsl:call-template name="recursive-handle-double-quotes">
			<xsl:with-param name="text" select="." />
		</xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template match="srgs:ruleref">
    <xsl:choose>
      <xsl:when test="string(@special)!=''"><xsl:call-template name="addweight"/> $<xsl:value-of select="@special"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="starts-with(string(@uri),'#')">
            $<xsl:value-of select="substring-after(@uri,'#')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:choose>
            <xsl:when test="string(@type)!=''">
              $<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@uri"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>~<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@type"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              $<xsl:text disable-output-escaping="yes">&lt;</xsl:text><xsl:value-of select="@uri"/><xsl:text disable-output-escaping="yes">&gt;</xsl:text>
            </xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text> </xsl:text>
  </xsl:template>

  <xsl:template match="srgs:one-of">
    (<xsl:apply-templates select="srgs:item"/>)<xsl:call-template name="addlang"/><xsl:call-template name="addtag"/>
  </xsl:template>

  <xsl:template match="srgs:one-of/srgs:item"><xsl:call-template name="addweight"/>(<xsl:apply-templates />)<xsl:call-template name="addlang"/><xsl:call-template name="addtag"/><xsl:call-template name="addrepeat"/><xsl:if test="not(position()=last())"> | </xsl:if></xsl:template>

  <xsl:template match="srgs:item">
    (<xsl:apply-templates/>)<xsl:call-template name="addlang"/><xsl:call-template name="addtag"/><xsl:call-template name="addrepeat"/>
  </xsl:template>

</xsl:stylesheet>


