#ifndef __LV_SRE_ADVANCED_H__F1FC6534_D093_452E_8CFF_3EEC50A05A00__INCLUDED__
#define __LV_SRE_ADVANCED_H__F1FC6534_D093_452E_8CFF_3EEC50A05A00__INCLUDED__

//prevent name-mangling
#if defined(__cplusplus)
extern "C"
{
#endif

//Library initialization and de-initialization calls.
LV_SRE_API void LV_SRE_Startup(void);
LV_SRE_API void LV_SRE_Shutdown(void);

LV_SRE_API float LV_SRE_DecodePitch(HPORT hport,int VoiceChannel);
LV_SRE_API float LV_SRE_DecodeEnergy(HPORT hport,int VoiceChannel);

LV_SRE_API int LV_SRE_GetVoiceChannelData(HPORT hport,int VoiceChannel, short** PCM,  
										  unsigned int* Samples);

LV_SRE_API int LV_SRE_IsGrammarLoaded(HPORT hport, const char* name);

LV_SRE_API int LV_SRE_GetDecodeMS(HPORT hport,int voicechannel);
LV_SRE_API int LV_SRE_GetTotalDecodeMS(HPORT hport,int voicechannel);

LV_SRE_API int LV_SRE_GetDecodeAcousticModel(HPORT hport,int vc,char *pzmodel,int buffersize);

LV_SRE_API int LV_SRE_GetLogFileName(HPORT hport,int voicechannel, char *FileName,int* FileNameStringSize);

LV_SRE_API int LV_SRE_AddEvent(HPORT hport, const char* eventName);
LV_SRE_API int LV_SRE_AddEventEx(HPORT hport, const char* eventName, const char* eventData);

// Adds a custom string to a named field in the call log file
LV_SRE_API int LV_SRE_AddFieldToRequest(HPORT hport, const char* KeyName, const char* String);

LV_SRE_API int LV_SRE_ClearAllFieldsToRequest(HPORT hport);

/* Get the language code of the grammar associated with a Semantic Data. 
 *
 * Input: hport			The port used in the decode.
 *        voicechannel	The audio channel that the port used during decode.
 *		  index			The index of the interpretation.  There is one semantic interpretation result
 *                      per parse tree, so the index ranges from 
 *					    0 to LV_SRE_GetNumberOfInterpretations - 1      
 */
LV_SRE_API const char* LV_SRE_GetInterpretationLanguage(HPORT hport, int VoiceChannel, int Index);

/* Get the mode of the grammar associated with a Semantic Data. 
 *
 * Input: hport			The port used in the decode.
 *        voicechannel	The audio channel that the port used during decode.
 *		  index			The index of the interpretation.  There is one semantic interpretation result
 *                      per parse tree, so the index ranges from 
 *					    0 to LV_SRE_GetNumberOfInterpretations - 1      
 */
LV_SRE_API const char* LV_SRE_GetInterpretationMode(HPORT hport, int VoiceChannel, int Index);

LV_SRE_API const char* LV_SRE_GetInterpretationTopLevelRule(HPORT hport, int VoiceChannel, int Index);
LV_SRE_API const char* LV_SRE_GetInterpretationInputString(HPORT hport, int VoiceChannel, int Index);

//call with each buffer of sound data of the prompt (playing in parralell to recorded sound data)
LV_SRE_API int LV_SRE_StreamSendPromptData(HPORT hport, void* SoundData, int SoundDataLength);

// Query status of stream (STREAM_STATUS_NOT_READY, STREAM_STATUS_READY, etc)
LV_SRE_API int LV_SRE_StreamGetStatus(HPORT hport);

// Query number of bytes of retained stream (does not count bytes before barge-in detection)
LV_SRE_API int LV_SRE_StreamGetLength(HPORT hport);

LV_SRE_API int LV_SRE_StreamGetParameter(HPORT hport, int StreamParameter, unsigned int* StreamParameterValue);
LV_SRE_API int LV_SRE_StreamSetParameterToDefault(HPORT hport, int StreamParameter);

/*
* Utility functions for confirming words exist in dictionary, generating 
* pronunciation of words
*/


/*
* Get number of alternate pronunciation for the words
*
*      PARAMETERS:
*          words			Space-separated words
*			lang	        The language of the dictionary for the search (e.g. en-US, es-MX...)
*
*		RETURN VALUES
*			> 0 - number of alternate pronunciation for the words
*			LV_INVALID_DICTIONARY_LANGUAGE: Invalid language specified
*      
*/
LV_SRE_API int LV_SRE_GetPhoneticPronunciationCount(const char *words, const char *lang);

/*
* Get the pronunciation of the words by pronunciation index
*
*      PARAMETERS:
*          words			Space-separated words
*			lang	        The language of the dictionary for the search (e.g.
*                          en-US, es-MX...)
*          idx             The index of the alternate pronunication
*          prounciation    The buffer to obtain the pronunciation of words
*          len             The size of the prounciation buffer
*
*		RETURN VALUES
*			LV_SUCCESS: success
*			LV_INVALID_DICTIONARY_LANGUAGE: Invalid language specified
*      
*/
LV_SRE_API int LV_SRE_GetPhoneticPronunciation(const char *words, const char *lang, int idx, char *pronunciation, int len);

/* Query the number of languages available to the SRE server */
LV_SRE_API int LV_SRE_GetAvailableLanguageCount();

/* Query the language string for the given index. Index must be less than the available count returned by above function */
LV_SRE_API int LV_SRE_GetAvailableLanguageIndex(int index, char *language, int length);

/* The following uses a handle to a grammar object created from the LV_SRE_Grammar API
(See LV_SRE_Grammar.h). */
LV_SRE_API int LV_SRE_LoadGrammarFromObject(HPORT hport, const char* gram, HGRAMMAR grammar);
LV_SRE_API int LV_SRE_LoadGlobalGrammarFromObject(const char* gram, HGRAMMAR grammar);


/* The following gets a handle to parse tree objects to be used in conjunction with the
LV_SRE_ParseTree API (See LV_SRE_ParseTree.h). The LV_SRE_ParseTree API can be used to
handle parse tree objects in the absense of an XML parser to parse the
output from LV_SRE_GetParseTreeString(). However, we recommend parsing the XML output */
LV_SRE_API H_PARSE_TREE LV_SRE_CreateParseTree(HPORT hport, int VoiceChannel, int Index);


/* The following gets handles to semantic data objects to be used in conjunction with the
LV_SRE_Semantic API (See LV_SRE_Semantic.h). The LV_SRE_Semantic API can be used to
handle semantic interpretation objects in the absense of an XML parser to parse the
output from LV_SRE_GetInterpretationString().  However, we recommend parsing the XML output */
LV_SRE_API H_SI LV_SRE_CreateInterpretation(HPORT hport, int voicechannel, int index);
LV_SRE_API H_SI_DATA LV_SRE_GetInterpretationData(HPORT hport, int VoiceChannel, int Index);

LV_SRE_API const char* LV_SRE_GetInterpretationPhonemes(HPORT hport, int VoiceChannel, int Index);

/* Returns 1 if successfully able to communicate with an SRE server in the connection list, otherwise 0 */
LV_SRE_API int LV_SRE_IsServerAvailable();

/* This function is used to set the ID used in call logging. If not used, a globally unique id
    will be generated automatically for each call */
LV_SRE_API int LV_SRE_SetCustomCallGuid(HPORT hport, const char* CallGuid);

/* This function is used to get the ID used in call logging. It can be used in conjuction with
    LV_TTS_SetCustomCallGuid() to link events across SRE ports and TTS ports to the same call log */
LV_SRE_API int LV_SRE_GetCallGuid(HPORT hport, char* CallGuid, int CallGuidLength);

#if defined(__cplusplus)
}
#endif

#endif	/* __LV_SRE_ADVANCED_H__F1FC6534_D093_452E_8CFF_3EEC50A05A00__INCLUDED__ */
