#ifndef __LV_SRE_DEFINES_H__A7F2A79F_229A_4FDB_8F8B_B1D6096079A4__INCLUDED__
#define __LV_SRE_DEFINES_H__A7F2A79F_229A_4FDB_8F8B_B1D6096079A4__INCLUDED__


#if !defined(NULL)
#define NULL 0
#endif

//================================================================
//=============== Defines from LV_SRE_Parse_Tree.h ===============
//================================================================

/* handle for a parse tree
A parse tree (or sentence diagram, if you prefer) is a tree structure that illustrates how a speech
utterance was matched against a grammar by the SRE.
Parse trees come from the speech port after a successful decode.
You destroy the handle when you are done with it.
*/
typedef void* H_PARSE_TREE;

/* handle for a node in a parse tree.  node handles are only
valid for the life of the tree they come from.  the tree creates them and
you are responsible for destroying them (do not call delete/free).
*/
typedef void const* H_PARSE_TREE_NODE;

/* handle for a tree (or subtree) iterator
an iterator starts at a node, and traverses through the entire subtree of that node
in a top-to-bottom and left-to-right fashion.  Each node in the subtree is visited
exactly once.
Its useful if you wish to iteratively travel through a tree.
You destroy the iterator handle when you are done with it.
*/
typedef void* H_PARSE_TREE_ITR;

/* handle for a children iterator.
A children iterator provides access to the children of a parse tree node.
Its useful if you wish to recursively travel through a tree.
You destroy the iterator handle when you are done with it.
*/
typedef void* H_PARSE_TREE_CHILDREN_ITR;


typedef void* H_PARSE_TREE_TAG_ITR;
typedef void* H_PARSE_TREE_TERMINAL_ITR;
typedef void* H_PARSE_TREE_CONCEPT_ITR;

/*
* handle for a Semantic Interpretation Object
*/
typedef void* H_SI;
/*
* handle for a Semantic Interpretation Data Object
*/
typedef void * H_SI_DATA;

#define SI_TYPE_NULL	0	//Usually this type indicates that some error occured.
#define SI_TYPE_BOOL	1
#define SI_TYPE_INT		2
#define SI_TYPE_DOUBLE	3
#define SI_TYPE_STRING	4
#define SI_TYPE_OBJECT	5
#define SI_TYPE_ARRAY	6

/*
* Semantic Data printing format
* Both formats are used by "http://www.w3.org/TR/semantic-interpretation/"
*/
#define SI_FORMAT_ECMA	0
#define	SI_FORMAT_XML	1

typedef void (*SIErrorCallback)(const char* msg, void *p);


//================================================================
//================ Defines from LV_SRE_Grammar.h =================
//================================================================

typedef void* HGRAMMAR;

typedef void (*GrammarLogCB)(const char* string, int error_level, void* user_data);


//================================================================
//==================== Defines from LV_SRE.h =====================
//================================================================

typedef enum { LV_FALSE=0,LV_TRUE } lv_bool ;

typedef void* HPORT;


//================================================================
//================== Defines for TTS interface ===================
//================================================================

typedef void* HTTSCLIENT;

typedef int LV_TTS_RETURN_CODE;

// Flags for LV_TTS_Synthesize()
#define LV_TTS_BLOCK                     0x00000001


//Error return codes //////////////////////////////////////////////////////////
#ifndef	LV_ERROR_CODES
#define LV_ERROR_CODES
#define		LV_SUCCESS								 0
#define		LV_FAILURE								-1
#define		LV_SYSTEM_ERROR							-2
#define		LV_RESET								-3
#define		LV_BAD_SOUND_DATA						-4
#define		LV_INVALID_SOUND_FORMAT					-5
#define		LV_TIME_OUT								-6
#define		LV_GRAMMAR_SET_OUT_OF_RANGE				-7
#define		LV_SOUND_CHANNEL_OUT_OF_RANGE			-8
#define		LV_STANDARD_GRAMMAR_ALREADY_LOADED		-9
#define		LV_STANDARD_GRAMMAR_OUT_OF_RANGE	   -10
#define     LV_INVALID_PROPERTY_VALUE              -11
#define     LV_INVALID_HPORT                       -12
#define     LV_NOT_IMPLEMENTED                     -13
#define     LV_SOCKETS_ERROR	                   -14
#define     LV_INVALID_PROPERTY_TARGET             -15
#define     LV_INVALID_PROPERTY_VALUE_TYPE         -16
#define     LV_INVALID_PROPERTY                    -17
#define     LV_INVALID_PROPERTY_TARGET_NDX         -18
#define     LV_STREAM_NOT_ACCEPTED				   -19
#define     LV_FUNCTION_NOT_FOUND				   -20
#define     LV_STRING_BUFFER_TOO_SMALL			   -21
#define		LV_NO_SERVER_AVAILABLE				   -22
#define		LV_GRAMMAR_SYNTAX_WARNING			   -23
#define		LV_GRAMMAR_SYNTAX_ERROR                -24
#define		LV_GRAMMAR_LOADING_ERROR               -25

#define		LV_OPEN_PORT_FAILED__LICENSES_EXCEEDED				-26
#define		LV_OPEN_PORT_FAILED__PRIMARY_SERVER_NOT_RESPONDING	-27
#define		LV_OPEN_PORT_FAILED__PRIMARY_SERVER_NOT_COMPATIBLE	-28
#define		LV_OPEN_PORT_FAILED__OPEN_PORT_NOT_CALLED_YET		-29

#define		LV_NO_SERVER_RESPONDING								-30

#define		LV_GLOBAL_GRAMMAR_TRANSACTION_PARTIAL_ERROR		    -31
#define		LV_GLOBAL_GRAMMAR_TRANSACTION_ERROR					-32

#define		LV_ACTIVE_GRAMMAR_VOCAB_SIZE_EXCEEDS_LIMIT			-33

#define     LV_INVALID_LICENSE                                  -34
#define		LV_ACTIVE_GRAMMAR_LANGUAGE_CONFLICT					-35
#define		LV_ACTIVE_GRAMMAR_INVALID_LANGUAGE					-36
#define     LV_LOAD_GRAMMAR_TIMEOUT                             -37

#define     LV_INVALID_DICTIONARY_LANGUAGE                      -38
#define     LV_WORDS_NOT_IN_DICTIONARY                          -39
#define     LV_LVLANG_UNDEFINED                                 -40

#define     LV_LICENSES_EXPIRED                                 -41


#define		LV_NO_ANSWERS_AVAILABLE								-42
#define		LV_INVALID_INDEX									-43
#define		LV_INVALID_GRAMMAR_LABEL							-44
#define		LV_INVALID_SOUND_CHANNEL							-45
#define		LV_INVALID_EVENT									-46
#define		LV_INVALID_BUFFER									-47
#define		LV_INVALID_GRAMMAR_OBJECT							-48
#define		LV_INCOMPATIBLE_SRE_SERVER							-49
#define		LV_UNKNOWN_SERVER_ERROR								-50

#define		LV_INVALID_TTS_HANDLE								-51	// Invalid TTS handle (external)
#define		LV_NO_TTS_SERVERS									-52	// No TTS servers available
#define		LV_INVALID_CLIENT_OBJECT							-53	// Invalid internal object
#define		LV_INVALID_API_PARAMETER							-54	// One or more parameters to an API function is/are invalid
#define		LV_NO_COMPATIBLE_TTS_SERVERS						-55	// Either no TTS servers active or none have the required capabilities
#define		LV_SSML_ERROR										-56	// SSML processing error
#define		LV_NO_RESULT_AVAILABLE								-57 // Returned whenever operations are performed before results become available

// License related errors
#define		LV_ACQUIRING_LICENSE_FAILED							-58	// Could not obtain a license
#define		LV_EXPIRED_LICENSE									-59	// Current license does not permit using the version of the TTS server
#define		LV_INVALID_LICENSE_MAINT_DATE						-60	// License maintenance expiry date is invalid, something is messed up

// TTS server side errors
#define		LV_RESOURCE_ERROR									-61	// Failed to allocate certain resources on the server in an effort to complete the requested operation
#define		LV_FAILED_VOICE_LOAD								-62	// Failed to load a required voice on the server
#define		LV_FAILED_NO_VOICE_SPECIFIED						-63	// No voices specified in request. At least one voice is needed to perform text-to-speech
#define		LV_FAILED_STREAMER									-64	// Failed to instantiate the synthesis object on the server
#define		LV_FAILED_SYNTHESIS									-65	// Actual synthesis itself failed on the server
#define		LV_FAILED_AUDIO_RESAMPLING							-66	// Resampling to the requested output sampling rate failed on the server
#define		LV_FAILED_AUDIO_CONVERSION							-67	// Conversion of the audio from the native form to the requested type failed on the server

#define		LV_EXCEPTION										-68 // Exception caught by try catch

#endif //LV_ERROR_CODES

//Used internally
#define     LV_LAST_ERROR LV_EXCEPTION
#define     LV_NUM_ERRORS ((-(LV_EXCEPTION))+1)

enum SOUND_FORMAT
{
	UNK_FORMAT = 0,
	ULAW_8KHZ,
	PCM_8KHZ,
	PCM_16KHZ,
	ALAW_8KHZ,
	SPX_8KHZ,
	SPX_16KHZ
};

enum SYNTHESIS_SOUND_FORMAT
{
	SFMT_UNKNOWN = 0,
	SFMT_ULAW,
	SFMT_PCM,
	SFMT_ALAW
};


//SetPropertyEx - properties
#define PROP_EX_TRIM_SILENCE_VALUE 1

#define PROP_EX_SAVE_SOUND_FILES   2
// Does not store CallSRE files
#define SAVE_SOUND_FILES_NONE			0
// Store all CallSRE data when decodes occur
#define SAVE_SOUND_FILES_BASIC			1
// Store all BASIC data plus the entire streaming data when StreamCancel is called which is assumed to coincide with BargeInTimeout
#define SAVE_SOUND_FILES_ADVANCED		2
// Stores all CallSRE data stored in Advanced in addition to the entire streaming data during all SRE decodes
#define SAVE_SOUND_FILES_ALL			3


#define PROP_EX_LANGUAGE           3

#define PROP_EX_SRE_SERVERS        4	//this is a client level property (PROP_EX_TARGET_CLIENT) format "IP:Port;IP:Port"

//properties for noise reduction
#define PROP_EX_NOISE_REDUCTION_ENABLE			5
// No Noise Reduction
#define NOISE_REDUCTION_NONE			0
// Noise Reduction Default
#define NOISE_REDUCTION_DEFAULT			1
// Noise Reduction Alternate
#define NOISE_REDUCTION_ALTERNATE		2
// Noise Reduction Adaptive
#define NOISE_REDUCTION_ADAPTIVE		3

#define PROP_EX_NOISE_REDUCTION_AUTO_ADAPTIVE	6

#define PROP_EX_NOISE_REDUCTION_ALPHA			7

#define PROP_EX_CHOOSE_MODEL	8

#define PROP_EX_SET_SERVER_IP	10	//server IP addresses to connect to (comma delimited list) first IP is master server
//valid only for PROP_EX_TARGET_CLIENT (client process)

#define PROP_EX_SET_SERVER_PORT 11	//server Port addresses (to IPs of PROP_EX_SET_SERVER_IP)
//valid only for PROP_EX_TARGET_CLIENT (client process)

#define PROP_EX_SEARCH_BEAM_WIDTH  12 // during decode, beam width controls how many paths are searched.
// narrower beams lead to faster searches, but with reduced accuracy.
// the larger the beam value, the narrower the beam.
// a beam-width value of 0.0 corresponds to a completely
// open search.  Default value is 1e-6
// NOTE: the user does not need to set this any more, see PROP_EX_DECODE_OPTIMIZATION

// These have no effect anymore
#define PROP_EX_CONCEPT_REPETITION_MIN 13
#define PROP_EX_CONCEPT_REPETITION_MAX 14



///////////////////////////////////////////////////////////////////////////////
///
/// \def PROP_EX_ENABLE_LATTICE_CONFIDENCE_SCORE
///
/// \target PROP_EX_TARGET_CLIENT
/// \target PROP_EX_TARGET_PORT
/// \target PROP_EX_TARGET_CHANNEL
///
/// \type   PROP_EX_VALUE_TYPE_INT
/// \type   PROP_EX_VALUE_TYPE_INT_PTR
///
/// \default 1 (on)
///
///////////////////////////////////////////////////////////////////////////////
#define PROP_EX_ENABLE_LATTICE_CONFIDENCE_SCORE 15

///////////////////////////////////////////////////////////////////////////////
///
/// \def PROP_EX_MAX_NBEST_RETURNED
///
/// \target PROP_EX_TARGET_CLIENT
/// \target PROP_EX_TARGET_PORT
/// \target PROP_EX_TARGET_CHANNEL
///
/// \type   PROP_EX_VALUE_TYPE_INT
/// \type   PROP_EX_VALUE_TYPE_INT_PTR
///
/// required to be an integer >= 1
///
/// \default 1
///
///////////////////////////////////////////////////////////////////////////////
#define PROP_EX_MAX_NBEST_RETURNED 16

///////////////////////////////////////////////////////////////////////////////
/// \def PROP_EX_DECODE_TIMEOUT
///
/// \target PROP_EX_TARGET_CLIENT
/// \target PROP_EX_TARGET_PORT
/// \target PROP_EX_TARGET_CHANNEL
///
/// \type PROP_EX_VALUE_TYPE_INT
/// \type PROP_EX_VALUE_TYPE_INT_PTR
///
/// This value is in second.
///
/// \default 20 (sec.)
///
///////////////////////////////////////////////////////////////////////////////
#define PROP_EX_DECODE_TIMEOUT  17

///////////////////////////////////////////////////////////////////////////////
/// \def Automatic acoustic model selecting thresholds
///
/// \target PROP_EX_TARGET_CLIENT
/// \target PROP_EX_TARGET_PORT
/// \target PROP_EX_TARGET_CHANNEL
///
/// \type PROP_EX_VALUE_TYPE_INT
/// \type PROP_EX_VALUE_TYPE_INT_PTR
///
/// \default LOW_THLD = 135, HIGH_THLD = 155
///
///////////////////////////////////////////////////////////////////////////////
#define PROP_EX_MOD_SEL_LOW_THLD	18
#define PROP_EX_MOD_SEL_HIGH_THLD	19

#define PROP_EX_FLAT_SEARCH	        20

#define PROP_EX_TREE_SEARCH 	    21

#define PROP_EX_WORDSIZE_AUTOPICK_SEARCH 	        22

#define PROP_EX_LICENSE_TYPE        23

///////////////////////////////////////////////////////////////////////////////
///
/// \def PROP_EX_DECODE_THREAD_PRIORITY
///
/// \target PROP_EX_TARGET_CLIENT
/// \target PROP_EX_TARGET_PORT
/// \target PROP_EX_TARGET_CHANNEL
///
/// \type   PROP_EX_VALUE_TYPE_INT
/// \type   PROP_EX_VALUE_TYPE_INT_PTR
///
/// required to be one of PROP_EX_DECODE_THREAD_PRIORITY_LOW, PROP_EX_DECODE_THREAD_PRIORITY_NORMAL or PROP_EX_DECODE_THREAD_PRIORITY_HIGH
///
/// \default PROP_EX_DECODE_THREAD_PRIORITY_NORMAL
///
///////////////////////////////////////////////////////////////////////////////
#define PROP_EX_DECODE_THREAD_PRIORITY 24

#define PROP_EX_DECODE_THREAD_PRIORITY_LOW		0
#define PROP_EX_DECODE_THREAD_PRIORITY_NORMAL	1
#define PROP_EX_DECODE_THREAD_PRIORITY_HIGH		2


#define PROP_EX_LOAD_GRAMMAR_TIMEOUT 25

// Used to set the decode optimization. See values below
#define PROP_EX_DECODE_OPTIMIZATION	26
//Makes decodes faster by narrowing the beam width. Possible negligible loss of accuracy
#define PROP_EX_MAXIMIZE_DECODE_SPEED		1
//Makes decodes more accurate by widening the beam width. Possible negligible additional delay
#define PROP_EX_MAXIMIZE_DECODE_ACCURACY	2
//Automatically switches between the above 2 modes depending upon load
#define PROP_EX_OPTIMIZE_DECODE_AUTO		0


// These two properties are suppressed in favor of PROP_EX_LICENSE_SERVERS.
// They will be removed in the future.
#define PROP_EX_LIC_SERVER_HOSTNAME		27
#define PROP_EX_LIC_SERVER_PORTNUM		28

#define PROP_EX_STRICT_SISR_COMPLIANCE	29

#define PROP_EX_ACOUSTIC_MODEL_RESOLUTION 30      //value among 0 ,1 2, the higher the value the slower the speed.
#define PROP_EX_LOW_RESOLUTION_ACOUSTIC_MODEL 0
#define PROP_EX_MIDDLE_RESOLUTION_ACOUSTIC_MODEL 1
#define PROP_EX_HIGH_RESOLUTION_ACOUSTIC_MODEL 2

#define PROP_EX_SPEED_VS_ACCURACY 31      //value among 0 ,1 2, the higher the value the slower the speed.

#define	PROP_EX_SPEED_VS_ACCURACY_DEFAULT_SPEED 50

#define	PROP_EX_SPEED_VS_ACCURACY_HIGHEST_ACCURACY 100

#define PROP_EX_BUILTIN_GRAMMAR_LANGUAGE    32

#define PROP_EX_LICENSE_SERVERS	            33

#define PROP_EX_SYNTH_SOUND_FORMAT			34

#define PROP_EX_SYNTH_PROSODY_PITCH         35

#define PROP_EX_SYNTH_PROSODY_CONTOUR       36

#define PROP_EX_SYNTH_PROSODY_RANGE         37

#define PROP_EX_SYNTH_PROSODY_RATE          38

#define PROP_EX_SYNTH_PROSODY_DURATION      39

#define PROP_EX_SYNTH_PROSODY_VOLUME        40

#define PROP_EX_SYNTH_VOICE_GENDER          41

#define PROP_EX_SYNTH_VOICE_AGE             42

#define PROP_EX_SYNTH_VOICE_VARIANT         43

#define PROP_EX_SYNTH_VOICE_NAME            44

#define PROP_EX_SYNTH_EMPHASIS_LEVEL        45

#define PROP_EX_LOG_TTS_EVENTS				46

#define PROP_EX_SYNTHESIS_LANGUAGE			47

#define PROP_EX_SYNTHESIS_SAMPLING_RATE		48
//
#define MAX_PROP_EX                         48 // Update this parameter when new properties are added

//SetPropertyEx - map old names to new names
#define PROP_TRIM_SILENCE_VALUE             PROP_EX_TRIM_SILENCE_VALUE
#define PROP_SAVE_SOUND_FILES               PROP_EX_SAVE_SOUND_FILES

//SetPropertyEx - port or channel
#define PROP_EX_TARGET_PORT                  1
#define PROP_EX_TARGET_CHANNEL				 2	// Now applies to PORT instead. Will be removed in the future
#define PROP_EX_TARGET_GRAMMAR               3	// Not valid anymore. Does not affect anything
#define PROP_EX_TARGET_CLIENT                4	//all ports            //set hport to null when using this parm type

//SetPropertyEx - indicates how to typecast void *
#define PROP_EX_VALUE_TYPE_INT               1
#define PROP_EX_VALUE_TYPE_INT_PTR           2
#define PROP_EX_VALUE_TYPE_STRING            3
#define PROP_EX_VALUE_TYPE_FLOAT_PTR         4
#define PROP_EX_VALUE_TYPE_FLOAT            PROP_EX_VALUE_TYPE_FLOAT_PTR /*misnamed.*/

//SetPropertyEx - Language names
#define PROP_EX_LANG_ENGLISH_AMERICAN          "EN-US"/*"AmericanEnglish"*/
#define PROP_EX_LANG_ENGLISH_BRITISH           "EN-GB" // british english
#define PROP_EX_LANG_ENGLISH_AUSTRALIAN        "EN-AU" //AUSTRALIAN ENGLISH
#define PROP_EX_LANG_SPANISH_MEXICAN           "ES-MX" /*"MEXICAN Spanish"*/
#define PROP_EX_LANG_SPANISH_COLUMBIAN         "ES-CO" /*"COLUMBIAN Spanish"*/
#define PROP_EX_LANG_FRENCH_CANADIAN           "FR-CA" /*"French CANADIAN"*/
#define PROP_EX_LANG_GERMAN_GERMAN             "DE-DE" /*"German */
//SetPropertyEx - digit language names
#define PROP_EX_LANG_SPANISH_MEXICAN_DIGITS    "ES-MX-DI" /*"MEXICAN Spanish digits"*/
#define PROP_EX_LANG_ENGLISH_AMERICAN_DIGITS   "EN-US-DI" /*"AMERICAN English digits"*/
#define PROP_EX_LANG_ENGLISH_AUSTRALIAN_DIGITS "EN-AU-DI" //Australian English Digits
#define PROP_EX_LANG_ENGLISH_BRITISH_DIGITS    "EN-GB-DI" //British English Digits
#define PROP_EX_LANG_FRENCH_CANADIAN_DIGITS    "FR-CA-DI" //FRENCH CANADIAN Digits
#define PROP_EX_LANG_SPANISH_COLUMBIAN_DIGITS  "ES-CO-DI" //SPANISH COLUMBIAN DIGITS
#define PROP_EX_LANG_ENGLISH_GERMAN_DIGITS     "DE-DE-DI" //German Digits

//SetPropertyEx - License names
#define PROP_EX_LICENSE_VOXLITE                "VoxLite"
#define PROP_EX_LICENSE_SPEECHPORT             "SpeechPort"
#define PROP_EX_LICENSE_AMD                    "AMD"
#define PROP_EX_LICENSE_SLM                    "SLM"

//bitflags - currently only used by Decode()
#define LV_DECODE_USE_OOV                      1
#define LV_DECODE_BLOCK                        2
#define LV_DECODE_GENDER_MALE                  4
#define LV_DECODE_GENDER_FEMALE                8
#define LV_DECODE_FIRST_TIME_USER             16
#define LV_DECODE_RETURN_EACH_DIGIT           32
// Specifies that all decodes on the port are for the same call. This allows for 
// certain settings such as noise estimates to be retained for consecutive decodes 
// on the same call.
#define LV_DECODE_CONTINUED_CALL              64 
//64 and 128 are reserved.
#define LV_DECODE_SRGS_GRAMMAR               256
#define LV_DECODE_SEMANTIC_INTERPRETATION    512

//used to identify the SRGS ActiveGrammarSet in the Decode's "grammarset" argument.
#define LV_ACTIVE_GRAMMAR_SET -1

//used as voicechannel argument to get answers back.
#define LV_DTMF_CHANNEL                        -1
#define LV_INTERPRET_CHANNEL                   -2
#define LV_BEEP_CHANNEL                        -3


//Event defines
#define EVENT_DTMF "DTMF"
#define EVENT_TRANSFER "TRANSFER"
#define EVENT_NOINPUT "NO_INPUT"
#define EVENT_START_DECODE_SEQ "START_DECODE_SEQ"
#define EVENT_END_DECODE_SEQ "END_DECODE_SEQ"
#define EVENT_SRE "SRE"
#define EVENT_SRE_FAIL "SRE_FAIL"
#define EVENT_SRE_APPLICATION_ACCEPTED_CONCEPT "SRE_APP_CONCEPT"
#define EVENT_DTMF_DECODE "DTMF_DECODE"
#define EVENT_AMD_BEEP "AMD_BEEP"
#define EVENT_TTS "TTS"
#define EVENT_START_TTS_SEQ	"START_TTS_SEQ"
#define EVENT_END_TTS_SEQ	"END_TTS_SEQ"


#define STREAM_STATUS_NOT_READY	                0 //must call LV_SRE_StreamStart function to set up new stream
#define STREAM_STATUS_READY                     1 //port can accept streamed data
#define STREAM_STATUS_BARGE_IN                  2 //barge-in detected, now saving streamed data (user code can stop playback of audio data)
#define STREAM_STATUS_END_SPEECH                3 //when returned , new stream data no longer accepted until one of the StartStream functions called
#define STREAM_STATUS_STOPPED                   4 //StreamStop was called,
#define STREAM_STATUS_BARGE_IN_TIMEOUT          5 //barge-in not triggered before timeout, no audio will be saved.
#define STREAM_STATUS_END_SPEECH_TIMEOUT        6 //end-speech not detected.  call StreamStop to force a decode if in AutoDecode mode.
#define STREAM_STATUS_BEEP						10 //a beep was detected in the stream

#define STREAM_STATUS_REALTIMERESULT           11 //a realtime result was returned
#define STREAM_STATUS_COMPILEDRESULT           12 //a compiled result was returned
#define STREAM_STATUS_HOTWORDRESULT            13 //a result in hotword mode was returned
#define STREAM_STATUS_SPEECH_MARKER            14 //a TTS Speech Marker was reached
#define STREAM_STATUS_SPEECH_IN_PROGRESS       15 //a TTS Speak task became IN-PROGRESS
#define STREAM_STATUS_FAX_BEEP                 16 //a fax was detected in the stream
#define STREAM_STATUS_SIT_BEEP                 17 //a fax was detected in the stream

//BEGIN stream parameters used by LV_SRE_StreamSetParameter(), LV_SRE_StreamGetParameter() and LV_SRE_StreamSetParameterToDefault

#define STREAM_PARM_SOUND_FORMAT                1 //specifies sound data type stream handles - uses SOUND_FORMAT enum - default is ULAW_8KHZ
#define STREAM_PARM_VOICE_CHANNEL               2 //voice channel to load sound data to on stream completion
#define STREAM_PARM_GRAMMAR_SET                 3 //grammar set to use with auto decode streams
#define STREAM_PARM_DECODE_FLAGS                4 //decode flags to send with auto decode streams
#define STREAM_PARM_USE_COMPRESSION             5 //use compression internally for sound data - default = 1
#define STREAM_PARM_STORE_PROMPT                6 //save streamed prompt data in log file - default = 0


//BEGIN  stream type parameters  0 = false, 1 = true	//////////////////////////////////////////////////////////////

//if true will discard stream data until barge-in detected
#define STREAM_PARM_DETECT_BARGE_IN           101

//if true will stop excepting stream data on end-of-speech detect and change stream status to STREAM_STATUS_END_SPEECH
//  if auto_decode true, will immediately begin decoding
#define STREAM_PARM_DETECT_END_OF_SPEECH      102

//if true will start decode immediately on end-of-speech detection or a call to StopStream()
#define STREAM_PARM_AUTO_DECODE               103

// amount of audio in milliseconds that can be fed without barge-in occurring, before the state of
// the stream changes to STREAM_STATUS_BARGE_IN_TIMEOUT
#define STREAM_PARM_BARGE_IN_TIMEOUT          104

// amount of recorded audio (post-bargein) in milliseconds that can be fed without end-of-speech
// triggering, before the stream state changes to STREAM_STATUS_END_SPEECH_TIMEOUT
#define STREAM_PARM_END_OF_SPEECH_TIMEOUT     105

// will trigger a callback when a beep is detected in the stream
// used to listen for answering machine beeps so a prompt can restart
#define STREAM_PARM_NOTIFY_OF_BEEPS           106


//END    stream type parameters



//BEGIN barge-in adjustment values
//#define STREAM_PARM_BARGE_IN_BEGIN_DELAY	201		//number of .1 seconds at beginning of stream to limit barge-in
#define STREAM_PARM_BARGE_IN_NOISE_COUNT_LOW_THRESHOLD 202 //strength of signal to trigger barge-in  (default is 75)
#define STREAM_PARM_BARGE_IN_DYNAMIC_ADJUST 203 //if value set to 1 will dynamically adjust barge-in threshold
//END   barge-in adjustment values


//BEGIN end-of-speech detection types

//used to tune end-of-speech detection , use STREAM_END_OF_SPEECH_DETECTION_ value from below
//	this parameter is not activated until StartStream() is called (changing it on the fly has no effect)
#define STREAM_PARM_END_OF_SPEECH_DETECTION	210
//speech sent to SRE is a mix of single words and longer utterances (default setting)
#define STREAM_END_OF_SPEECH_DETECTION_NORMAL				0
//speech is just single words, end-of-speech detection is faster
//	(SRE does not need to wait for more words to follow)
#define STREAM_END_OF_SPEECH_DETECTION_SINGLE_WORDS			1
//speech is phrases or sentences, with potential pauses by speaker
//	(SRE will wait slightly longer before determining end-of-speech)
#define STREAM_END_OF_SPEECH_DETECTION_PHRASES_WITH_PAUSES	2
//speech is phrases or sentences, no pauses by speaker
#define STREAM_END_OF_SPEECH_DETECTION_PHRASES_NO_PAUSES	3

//older version had this define with too many underscores
//this is here so old
#define STREAM__END_OF_SPEECH_DETECTION_PHRASES_NO_PAUSES	STREAM_END_OF_SPEECH_DETECTION_PHRASES_NO_PAUSES


//END   end-of-speech detection types


// *** DEPRECATED ***
#define STREAM_PARM_NOISE_REDUCTION		220

// *** DEPRECATED ***
#define STREAM_PARM_USE_FREQ_VAD		221

// *** DEPRECATED ***
/////////////////////////////////////////////////////////////////////
///\brief Over all bargin threshold.
///
///range: (0,1000)
///
///default: 30
///
///This controls the over all sensitivity of the VAD. The lower this value is,
///the easier an audio frame gets classified as speech.
///////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_BARGEIN_LVL	222

//////////////////////////////////////////////////////////////////////
///\brief Time delay between detecting end-of-speech(EOS) and notifying EOS
///
///range: >0
///
///default: 800 ms
///
///In order to prevent cutting off weak EOS, once the VAD module detects EOS,
///it will wait for STREAM_PARM_VAD_EOS_DELAY ms before it notify the user.
////////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_EOS_DELAY	223

// *** DEPRECATED ***
///////////////////////////////////////////////////////////////////////
///\brief A audio frame is classified as speech only when the average energy
///on all frequency band exceeds this threshold.
///
///range: >0
///
///default: 200
///
///For noisy environment, raise this threshold to prevent false bargein caused by noise.
///////////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_NOISE_FLOOR 224

// *** DEPRECATED ***
////////////////////////////////////////////////////////////////////
///\brief peak-to-average ratio for current frame.
///
///range: [0~1000)
///
///default: 100
///
///For an audio frame that contains speech, voice frequency band usually features
///higher energy than other frequency bands that don't have speech signal. So, high quality
///speech frame  usually presents a high peak-to-average ratio. Raise this threshold to
///prevent false bargein caused by non-speech frame that presenting a relative high peak-to-average ratio
/////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_P2A_THLD	225


////////////////////////////////////////////////////////////////////
///\brief VAD volume sensitivity setting
///
///range: [0-100)
///
///default: 50
///
///VAD rejects frames below this sensitivity level based on volume.
///The value is mapped to an exponential scale in the backend
#define STREAM_PARM_VAD_VOLUME_SENSITIVITY 289


////////////////////////////////////////////////////////////////////
///\brief VAD SNR sensitivity setting
///
///range: [0-100)
///
///default: 50
///
///VAD rejects frames below this sensitivity level based on SNR.
#define STREAM_PARM_VAD_SNR_SENSITIVITY 290


////////////////////////////////////////////////////////////////////
///\brief VAD speech sensitivity setting
///
///range: [0-100)
///
///default: 50
///
///A higher value makes the VAD more sensitive to speech which means that the VAD is very 
///sure the data is speech before barge in. Raising the value will reject more false positives/noises
///However, it may mean that some speech that is on the borderline may be rejected
///This value should not be changed from the default without significant tuning and verification
////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_BARGEIN_THRESHOLD 291


//////////////////////////////////////////////////////////////////
///\brief wind-back length at the beginning of barge-in.
///
///range: >0
///
///default: 250ms
///
///The weak speech onset usually doesn't trigger barge-in. To compensate this, when the VAD detect barge-in,
///it will wind back a little to make sure that the beginning part of the speech remains in the buffer.
/////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_WIND_BACK	226

// *** DEPRECATED ***
//////////////////////////////////////////////////////////////////
///\brief The minimum duration of speech to trigger barge-in.
///
///range: >0
///
///default: 100ms
///
///To prevent speech-like bursting sound, such as cough, from triggering barge-in,
///The speech duration has to exceed this threshold to be classified as barge-in.
/////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_BURST_THLD	228

// *** DEPRECATED ***
//////////////////////////////////////////////////////////////////
///\brief The length of audio the VAD module uses to estimate the acoustic environment.
///
///range: >0
///
///default: 100ms
///
///Accurate VAD depends on good estimation of acoustic environment. The VAD module uses
///the first a couple of frames of audio to estimate the acoustic environment, such as noise level.
///The length of this period is defined by this parameter.
/////////////////////////////////////////////////////////////////////
//#define STREAM_PARM_VAD_INIT_TIME	229
//Use STREAM_PARM_VAD_STREAM_INIT_DELAY instead.



// *** DEPRECATED ***
//////////////////////////////////////////////////////////////////
///\brief Whether use zero-crossing criteria.
///
///value: 0 or 1.
///
///default: 0
///
///For mechanic like sound, sometimes the best solution to discriminate it from speech
///is using zero-crossing threshold on waveform. Set this parameter to 1 to turn on
///this functionality.
////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_USE_ZERO_CROSSING	230

// *** DEPRECATED ***
////////////////////////////////////////////////////////////////////
///\brief use band-passed audio signal for VAD.
///
///value: 0 or 1.
///
///default: 0
///
///By default, we consider audio frequency range [64HZ, KHZ], K is equal to half of sample rate.
///However most speech signal concentrates on the range [64HZ, 1000HZ]. So when high frequency noise
///exists, we better off ignore the signal in high frequency band. Set this parameter to 1
///to turn on this functionality.
////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_USE_BAND_PASS		231

////////////////////////////////////////////////////////////////////
///\brief Recognition mode.
///
///value: STREAM_PARM_RECOGNITION_MODE_NORMAL, STREAM_PARM_RECOGNITION_MODE_CONTRECOREALTIME, STREAM_PARM_RECOGNITION_MODE_CONTRECOCOMPILED or STREAM_PARM_RECOGNITION_MODE_HOTWORD.
///
///default: STREAM_PARM_RECOGNITION_MODE_NORMAL
///
///Defines the mode of recognition.
////////////////////////////////////////////////////////////////////
#define STREAM_PARM_RECOGNITION_MODE		232

#define STREAM_PARM_RECOGNITION_MODE_NORMAL				0

#define STREAM_PARM_RECOGNITION_MODE_CONTRECOREALTIME	1

#define STREAM_PARM_RECOGNITION_MODE_CONTRECOCOMPILED	2

#define STREAM_PARM_RECOGNITION_MODE_HOTWORD			3


#define STREAM_PARM_HOTWORD_MODE_CALLBACK_TRIGGER_SCORETHLD	233

// *** DEPRECATED ***
////////////////////////////////////////////////////////////////////
///\brief VAD init mode.
///
///value: STREAM_PARM_VAD_INIT_MODE_SILENCE_TRIMMED or STREAM_PARM_VAD_INIT_MODE_SILENCE_UNTRIMMED.
///
///default: STREAM_PARM_VAD_INIT_MODE_SILENCE_UNTRIMMED
///
///If STREAM_PARM_VAD_INIT_MODE_SILENCE_TRIMMED sets the VAD init time to 100ms.
///If STREAM_PARM_VAD_INIT_MODE_SILENCE_UNTRIMMED sets the VAD init time to 1000ms.
////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_INIT_MODE	234
#define STREAM_PARM_VAD_INIT_SILENCE_TRIMMED	0
#define STREAM_PARM_VAD_INIT_SILENCE_UNTRIMMED	1

/////////////////////////////////////////////////////////////////////
///\brief Use Echo Cancellation.
///
///range: 0 = OFF, 1 = ON
///
///default: 0
///
///Use Echo Cancellation.
///////////////////////////////////////////////////////////////////////
#define STREAM_PARM_USE_ECHO_CANCELLATION				235
#define STREAM_PARM_USE_ECHO_CANCELLATION_ON				1
#define STREAM_PARM_USE_ECHO_CANCELLATION_OFF				0
/////////////////////////////////////////////////////////////////////
///\brief Echo Cancellation convergence speed.
///
///range: (0,1000)
///
///default: 500
///
///This controls the convergence speech of the echo canceler. The higher the value, faster the convergence.
///However, there is a higher risk of instability as well as a larger amount of residual error
///after convergence.
///////////////////////////////////////////////////////////////////////
#define STREAM_PARM_ECHO_CANCELLATION_CONVERGERNCE		236

///////////////////////////////////////////////////////////////////////
///\brief Maximum Echo Delay expected in milliseconds.
///
///range: (20,500)
///
///default: 100
///
///Defines the maximum expected echo delay.
///////////////////////////////////////////////////////////////////////////
#define STREAM_PARM_MAX_ECHO_DELAY	237


//////////////////////////////////////////////////////////////////
///\brief The length of audio the VAD module uses to estimate the acoustic environment.
///
///range: >0
///
///default: 100ms
///
///Accurate VAD depends on good estimation of acoustic environment. The VAD module uses
///the first a couple of frames of audio to estimate the acoustic environment, such as noise level.
///The length of this period is defined by this parameter.
/////////////////////////////////////////////////////////////////////
#define STREAM_PARM_VAD_STREAM_INIT_DELAY	238


#define TIMEOUT_INFINITE		0

typedef void (*ExportLogMsg)(const char* String, void* p);

typedef void LV_SRE_StreamStateChangeFn(int NewState, unsigned int TotalBytes, unsigned int RecordedBytes, void* UserData);

typedef void LV_SRE_VAD_FE_CallbackFn(const char* Features, int StartTime, int EndTime, void* UserData);



#endif	/* __LV_SRE_DEFINES_H__A7F2A79F_229A_4FDB_8F8B_B1D6096079A4__INCLUDED__ */

