#ifndef __LV_SRE_DEPRECATED_H__
#define __LV_SRE_DEPRECATED_H__

#include <LV_SRE_Defines.h>

#ifdef LV_SRE_EXPORTS
#ifdef WIN32
#define LV_SRE_API __declspec(dllexport)
#define LV_SRE_LOCAL
#else
#define LV_SRE_API __attribute__ ((visibility ("default")))
#define LV_SRE_LOCAL __attribute__ ((visibility ("hidden")))
#endif
#else
#ifdef WIN32
#define LV_SRE_API __declspec(dllimport)
#define LV_SRE_LOCAL
#else
#define LV_SRE_API 
#define LV_SRE_LOCAL __attribute__ ((visibility ("hidden")))
#endif
#endif

#ifdef WIN32
#define LV_SRE_DEPRECATED(str)	__declspec(deprecated(str))
#define LV_SRE_REMOVED(str)	LV_SRE_DEPRECATED(str)
#else
#define LV_SRE_DEPRECATED(str)	__attribute__ ((__deprecated__))
#define LV_SRE_REMOVED(str)	__attribute__ ((__warning__ (str)))
#endif

#define LV_SRE_FN_RM(_api)		LV_SRE_REMOVED("The API function will be removed in future versions. Currently, calling it produces no effect and it simply returns.")
#define LV_SRE_FN_DE(_api)		LV_SRE_DEPRECATED("The API function is deprecated. Use " #_api " instead.")
#define LV_SRE_TY_RM(_api)		LV_SRE_REMOVED("The type will be removed in future versions. Use of this type is discouraged.")
#define LV_SRE_TY_DE(_api)		LV_SRE_DEPRECATED("The type is deprecated. Use " #_api " instead.")
#define LV_SRE_CL_RM(_api_c)	LV_SRE_REMOVED("This class of API functions is no longer supported.")
#define LV_SRE_CL_DE(_api_c)	LV_SRE_DEPRECATED("This class of API functions are deprecated in favor of the functions " #_api_c ".")


#ifdef WIN32
#pragma warning(push)
#endif

#ifdef __cplusplus
extern "C"
{
#endif

/* Overview:
* An OpenPort function is called to obtain a port handle (HPORT), 
* which is requrired for any SRE functionality
* LV_SRE_ClosePort is used to release the opened port handle
*/

/* LV_SRE_OpenPort/LV_SRE_OpenPort2 -- obtains a port handle, optionally registering a logging callback
 ExportLogMsg Log : Pointer to a user defined function which will receive logging information
 void* p : Pointer to user defind data will be passed into the ExportLogMsg function
 int verbosity : Should be an value between 0 - 6. 0 - minimal logging; 6 - maximum logging
 int* ErrorReturnCode : Pointer to a value, which can be used with LV_SRE_ReturnErrorString
                         to obtain a message indicating why an error occurred	
 Return Value: HPORT == NULL: Speechport error; HPORT != NULL: success */
LV_SRE_FN_DE("LV_SRE_CreateClient") LV_SRE_API HPORT LV_SRE_OpenPort(ExportLogMsg log,void *p,int verbosity);

LV_SRE_FN_DE("LV_SRE_CreateClient") LV_SRE_API HPORT LV_SRE_OpenPort2(int* ErrorReturnCode, ExportLogMsg log,void *p, int verbosity);


/* LV_SRE_ClosePort -- releases previously opened port handle
 HPORT hport : the port handle to be released, obtained from a previous call to LV_SRE_OpenPort
 Return Value: LV_SUCCESS No errors; the port has successfully shut down; LV_INVALID_HPORT The port was unable to shutdown*/ 
LV_SRE_FN_DE("LV_SRE_DestroyClient") LV_SRE_API int LV_SRE_ClosePort(HPORT hport);

#ifdef __cplusplus
}
#endif

#ifdef WIN32
#pragma warning(pop)
#endif


#ifndef IGNORE_LV_DEPRECATED
#define DEPRECATED_LVSPEECHPORT_METHODS                                                                  \
public:                                                                                                  \
	int OpenPort(ExportLogMsg Log = NULL, void* p = NULL, int NewMsgVerbosity = 0);                      \
	int ClosePort();                                                                                     \
	void SetHPort(HPORT port);                                                                          
#endif	// IGNORE_LV_DEPRECATED


#endif	//__LV_SRE_DEPRECATED_H__
