#ifndef LV_SRE_PARSE_TREE_H
#define LV_SRE_PARSE_TREE_H

#include <LV_SRE_Defines.h>
#include "LV_SRE_Deprecated.h"

/**
 *	C interface for viewing detailed information about how the recognition engine parsed speech input.
 */


#ifdef __cplusplus
extern "C"
{
#endif

/* parse tree memory allocation, deallocation */
LV_SRE_API H_PARSE_TREE					LVParseTree_Create (void);
LV_SRE_API H_PARSE_TREE					LVParseTree_CreateFromCopy (H_PARSE_TREE Other);
LV_SRE_API void							LVParseTree_Copy (H_PARSE_TREE Tree, H_PARSE_TREE Other);
LV_SRE_API void							LVParseTree_Release (H_PARSE_TREE Tree);

/* access to iterators over the speech parse tree. */
LV_SRE_API H_PARSE_TREE_ITR				LVParseTree_CreateIteratorBegin (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_ITR				LVParseTree_CreateIteratorEnd (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_CONCEPT_ITR     LVParseTree_CreateConceptIteratorBegin (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_CONCEPT_ITR     LVParseTree_CreateConceptIteratorEnd (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_TERMINAL_ITR    LVParseTree_CreateTerminalIteratorBegin (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_TERMINAL_ITR    LVParseTree_CreateTerminalIteratorEnd (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_TAG_ITR         LVParseTree_CreateTagIteratorBegin (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_TAG_ITR         LVParseTree_CreateTagIteratorEnd (H_PARSE_TREE Tree);

/* access to other speech parse tree data */
LV_SRE_API const char*					LVParseTree_GetTagFormat (H_PARSE_TREE Tree);
LV_SRE_API int							LVParseTree_GetNumberOfTagsInHeader (H_PARSE_TREE Tree);
LV_SRE_API const char*					LVParseTree_GetHeaderTag (H_PARSE_TREE Tree, int index);
LV_SRE_API const char*					LVParseTree_GetGrammarLabel (H_PARSE_TREE Tree);
LV_SRE_API const char*					LVParseTree_GetLanguage (H_PARSE_TREE Tree);
LV_SRE_API const char*					LVParseTree_GetMode (H_PARSE_TREE Tree);
LV_SRE_API H_PARSE_TREE_NODE			LVParseTree_GetRoot (H_PARSE_TREE Tree);
LV_SRE_API int                          LVParseTree_GetScore (H_PARSE_TREE Tree);

/* Returns true (non-zero) if the trees 
 *
 * 1. are the same language, mode, and have the same tag-format
 * 2. each node is equal and each node has the same number of children and they are equal
 * 
 * For the nodes, it ignores pronunciation, start times, end times, and scores.
 * This is so that the equality operator can be used to test
 * trees generated by the speechport against trees generated by grammar + transcripts.
 */
LV_SRE_API int							LVParseTree_AreEqual (H_PARSE_TREE Tree1, H_PARSE_TREE Tree2);

/*	iterator methods.  Access to what the iterator is sitting over, as well as the ability
	to advance the iterator's position
 */
LV_SRE_API H_PARSE_TREE_ITR				LVParseTree_Iterator_Create(void);
LV_SRE_API H_PARSE_TREE_ITR				LVParseTree_Iterator_CreateFromCopy(H_PARSE_TREE_ITR Other);
LV_SRE_API void							LVParseTree_Iterator_Copy(H_PARSE_TREE_ITR Iterator, H_PARSE_TREE_ITR Other);
LV_SRE_API void							LVParseTree_Iterator_Release (H_PARSE_TREE_ITR Iterator);
LV_SRE_API void							LVParseTree_Iterator_Advance (H_PARSE_TREE_ITR Iterator);
LV_SRE_API int							LVParseTree_Iterator_IsPastEnd (H_PARSE_TREE_ITR Iterator);
LV_SRE_API int							LVParseTree_Iterator_AreEqual (H_PARSE_TREE_ITR Iterator, H_PARSE_TREE_ITR Other);
LV_SRE_API H_PARSE_TREE_NODE			LVParseTree_Iterator_GetNode (H_PARSE_TREE_ITR Iterator);
LV_SRE_API void							LVParseTree_Release_Node(H_PARSE_TREE_NODE N);


LV_SRE_API H_PARSE_TREE_CHILDREN_ITR	LVParseTree_ChildrenIterator_Create (void);
LV_SRE_API H_PARSE_TREE_CHILDREN_ITR	LVParseTree_ChildrenIterator_CreateFromCopy (H_PARSE_TREE_CHILDREN_ITR Other);
LV_SRE_API void							LVParseTree_ChildrenIterator_Copy (H_PARSE_TREE_CHILDREN_ITR Iterator, H_PARSE_TREE_CHILDREN_ITR Other);
LV_SRE_API void							LVParseTree_ChildrenIterator_Release (H_PARSE_TREE_CHILDREN_ITR Iterator);
LV_SRE_API void							LVParseTree_ChildrenIterator_Advance (H_PARSE_TREE_CHILDREN_ITR Iterator);
LV_SRE_API int							LVParseTree_ChildrenIterator_IsPastEnd (H_PARSE_TREE_CHILDREN_ITR Iterator);
LV_SRE_API int							LVParseTree_ChildrenIterator_AreEqual (H_PARSE_TREE_CHILDREN_ITR Iterator, H_PARSE_TREE_CHILDREN_ITR Other);
LV_SRE_API H_PARSE_TREE_NODE			LVParseTree_ChildrenIterator_GetNode (H_PARSE_TREE_CHILDREN_ITR Iterator);


LV_SRE_API H_PARSE_TREE_TERMINAL_ITR	LVParseTree_TerminalIterator_Create (void);
LV_SRE_API H_PARSE_TREE_TERMINAL_ITR	LVParseTree_TerminalIterator_CreateFromCopy (H_PARSE_TREE_TERMINAL_ITR Other);
LV_SRE_API void							LVParseTree_TerminalIterator_Copy (H_PARSE_TREE_TERMINAL_ITR Iterator, H_PARSE_TREE_TERMINAL_ITR Other);
LV_SRE_API void							LVParseTree_TerminalIterator_Release (H_PARSE_TREE_TERMINAL_ITR Iterator);
LV_SRE_API void							LVParseTree_TerminalIterator_Advance (H_PARSE_TREE_TERMINAL_ITR Iterator);
LV_SRE_API int							LVParseTree_TerminalIterator_IsPastEnd (H_PARSE_TREE_TERMINAL_ITR Iterator);
LV_SRE_API int							LVParseTree_TerminalIterator_AreEqual (H_PARSE_TREE_TERMINAL_ITR Iterator, H_PARSE_TREE_TERMINAL_ITR Other);
LV_SRE_API H_PARSE_TREE_NODE			LVParseTree_TerminalIterator_GetNode (H_PARSE_TREE_TERMINAL_ITR Iterator);


LV_SRE_API H_PARSE_TREE_TAG_ITR			LVParseTree_TagIterator_Create (void);
LV_SRE_API H_PARSE_TREE_TAG_ITR			LVParseTree_TagIterator_CreateFromCopy (H_PARSE_TREE_TAG_ITR Other);
LV_SRE_API void							LVParseTree_TagIterator_Copy (H_PARSE_TREE_TAG_ITR Iterator, H_PARSE_TREE_TAG_ITR Other);
LV_SRE_API void							LVParseTree_TagIterator_Release (H_PARSE_TREE_TAG_ITR Iterator);
LV_SRE_API void							LVParseTree_TagIterator_Advance (H_PARSE_TREE_TAG_ITR Iterator);
LV_SRE_API int							LVParseTree_TagIterator_IsPastEnd (H_PARSE_TREE_TAG_ITR Iterator);
LV_SRE_API int							LVParseTree_TagIterator_AreEqual (H_PARSE_TREE_TAG_ITR Iterator, H_PARSE_TREE_TAG_ITR Other);
LV_SRE_API H_PARSE_TREE_NODE			LVParseTree_TagIterator_GetNode (H_PARSE_TREE_TAG_ITR Iterator);


LV_SRE_API H_PARSE_TREE_CONCEPT_ITR		LVParseTree_ConceptIterator_Create (void);
LV_SRE_API H_PARSE_TREE_CONCEPT_ITR		LVParseTree_ConceptIterator_CreateFromCopy (H_PARSE_TREE_CONCEPT_ITR Other);
LV_SRE_API void							LVParseTree_ConceptIterator_Copy (H_PARSE_TREE_CONCEPT_ITR Iterator, H_PARSE_TREE_CONCEPT_ITR Other);
LV_SRE_API void							LVParseTree_ConceptIterator_Release (H_PARSE_TREE_CONCEPT_ITR Iterator);
LV_SRE_API void							LVParseTree_ConceptIterator_Advance (H_PARSE_TREE_CONCEPT_ITR Iterator);
LV_SRE_API int							LVParseTree_ConceptIterator_IsPastEnd (H_PARSE_TREE_CONCEPT_ITR Iterator);
LV_SRE_API int							LVParseTree_ConceptIterator_AreEqual (H_PARSE_TREE_CONCEPT_ITR Iterator, H_PARSE_TREE_CONCEPT_ITR Other);
LV_SRE_API H_PARSE_TREE_NODE			LVParseTree_ConceptIterator_GetNode (H_PARSE_TREE_CONCEPT_ITR Iterator);

/* Methods for accessing data from a node, and for getting
   subtree and children iterators from that node.
 */
LV_SRE_API int							LVParseTree_Node_IsTag (H_PARSE_TREE_NODE Node);			
LV_SRE_API int							LVParseTree_Node_IsTerminal (H_PARSE_TREE_NODE Node);	
LV_SRE_API int							LVParseTree_Node_IsRule (H_PARSE_TREE_NODE Node);
LV_SRE_API int							LVParseTree_Node_GetLevel (H_PARSE_TREE_NODE Node);
LV_SRE_API int							LVParseTree_Node_GetScore (H_PARSE_TREE_NODE Node);
LV_SRE_API int							LVParseTree_Node_GetStartTime (H_PARSE_TREE_NODE Node);
LV_SRE_API int							LVParseTree_Node_GetBeginFrame (H_PARSE_TREE_NODE Node);
LV_SRE_API int							LVParseTree_Node_GetEndTime (H_PARSE_TREE_NODE Node);
LV_SRE_API int							LVParseTree_Node_GetEndFrame (H_PARSE_TREE_NODE Node);
LV_SRE_API const char*					LVParseTree_Node_GetPhonemes (H_PARSE_TREE_NODE Node);
LV_SRE_API const char*					LVParseTree_Node_GetText (H_PARSE_TREE_NODE Node);
LV_SRE_API const char*					LVParseTree_Node_GetRawText (H_PARSE_TREE_NODE Node);
LV_SRE_API const char*					LVParseTree_Node_GetRuleName (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_ITR				LVParseTree_Node_CreateIteratorBegin (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_ITR				LVParseTree_Node_CreateIteratorEnd (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_CHILDREN_ITR	LVParseTree_Node_CreateChildrenIteratorBegin (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_CHILDREN_ITR	LVParseTree_Node_CreateChildrenIteratorEnd (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_TERMINAL_ITR	LVParseTree_Node_CreateTerminalIteratorBegin (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_TERMINAL_ITR	LVParseTree_Node_CreateTerminalIteratorEnd (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_TAG_ITR			LVParseTree_Node_CreateTagIteratorBegin (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_TAG_ITR			LVParseTree_Node_CreateTagIteratorEnd (H_PARSE_TREE_NODE Node);
LV_SRE_API H_PARSE_TREE_NODE			LVParseTree_Node_CreateCopy (H_PARSE_TREE_NODE Node);

/* Returns true (non-zero) if the nodes are the same type,
 * And have the same rulename/text
 * Ignores pronunciation, start time, end time, and score.
 * This is so that the equality operator can be used to test
 * trees generated by the speechport against trees generated by grammar + transcripts.
 */
LV_SRE_API int                          LVParseTree_Node_AreEqual (H_PARSE_TREE_NODE Node1, H_PARSE_TREE_NODE Node2);


#ifdef __cplusplus
}
#endif

#ifdef __cplusplus

class LVParseTree_Iterator; //Also known as LVParseTree::Iterator
class LVParseTree_ChildrenIterator; //Also known as LVParseTree::ChildrenIterator
class LVParseTree_TagIterator; //Also known as LVParseTree::TagIterator
class LVParseTree_TerminalIterator; //Also known as LVParseTree::TerminalIterator
class LVParseTree_ConceptIterator; //Also known as LVParseTree::ConceptIterator

///////////////////////////////////////////////////////////////////////////////////////////////////
// 
// class LVParseTree_Node (also known as LVParseTree::Node)
//
// This is a node in a parse tree.
// Nodes can represent:
//
// 1. Rules:     Rules represent a matched grammar rule.
//               The children below this node represent the data that caused this rule 
//               to match.
//               The name of this rule can be accessed by calling Name().
//               The partial utterance that matched this rule can be accessed by 
//               calling RawText()
//               The Pronunciation of those words can be accessed by calling Phonemes();
//               The confidence of this rule can be accessed by calling Score();
//               The first audio frame for this rule can be accessed by calling BeginFrame();
//               The last audio frame for this rule can be accessed by calling
//
// 2. Terminals: Terminal nodes represent the words that the SRE recognized.
//               They are always leaf nodes in the parse tree.
//
// 3. Tags:      Tag nodes represent tag data in the grammar that was seen during the utterance.
//               Tag data can be anything, but usually represents meaning (semantic information).
//               Tags are always leaf nodes in the parse tree.
//
// Nodes are valid only for as long as the tree that created them is valid.
//
//////////////////////////////////////////////////////////////////////////////////////////////////
class LVParseTree_Node 
{
public:
	bool                          IsRule() const;
	bool                          IsTerminal() const;
	bool                          IsTag() const;

	//Tells you at what level in the tree this node is.  
	//The root of the tree is at level 0.  The root's children are at level 1, and so on.
	int                           Level() const;

	// For a rule, RuleName() tells you the name of the rule.
	// Otherwise, this is the name of the rule that contains this node.
	const char*                   RuleName() const;

	// For a rule, this is the partial sentence that caused the rule to be matched.
	// For a terminal, this is the word that was spoken.
	// For a tag, this is the tag data.
	// The difference between Text() and RawText() is that Text() gives you the text as it appears in the
	// grammar (case-sensitive), and RawText() gives you the text as it appears in the SRE (all-caps)
	const char*                   Text() const;
	const char*                   RawText() const;

	// For a rule, this is the phonetic pronunciation of the partial sentence that caused the rule
	// to be matched.
	// For a terminal, this is the pronunciation of the word that was spoken.
	// For a tag, this is empty.
	const char*                   Phonemes() const;

	bool operator==(const LVParseTree_Node& other) const;
	bool operator!=(const LVParseTree_Node& other) const {return !operator==(other);}
    
	//////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// coming soon:
	//   const char* Phrase() const; //a name that matches our current notation
	// or
	//   const char* RuleExpansion() const; //a name that matches the SRGS notation
	//
	// For a rule, this is the left-hand-side of the rule.
	// For a tag or terminal, this is the left-hand-side of the rule that contains this node.
	//
	//////////////////////////////////////////////////////////////////////////////////////////////////

	// For a rule, this is the confidence of this rule being matched (if I can figure out something sensible).
	// For a terminal, this is the word score.
	// For a tag, this is the parent rule's score.
	int                           Score() const;

	// For a rule, this is the beginning frame/start time of the first word that caused this rule to be matched.
	// For a terminal, this is the first frame/statt time of the matched word.
	// For a tag, this is the start time of the first word after it/the end time of the last word before it 
	int                           StartTime() const; //milliseconds
	int                           BeginFrame() const;//SRE audio frames 

	// For a rule, this is the last frame/end time of the last word that caused this rule to be matched.
	// For a terminal, this is the last frame of the matched word.
	// For a tag, this is the start time of the first word after it/the end time of the last word before it
	int                           EndTime() const;
	int                           EndFrame() const;

	// Returns an iterator that traverses the subtree of this node, beginning at this node.
	LVParseTree_Iterator         SubTreeBegin() const;
	LVParseTree_Iterator         SubTreeEnd() const;

	// Returns an iterator that traverses this nodes immediate children.
	LVParseTree_ChildrenIterator ChildrenBegin() const;
	LVParseTree_ChildrenIterator ChildrenEnd() const;

	//returns all the terminals under a rule node 
	//(equivalent to using a subtree iterator, and screening out non-terminals)
	LVParseTree_TerminalIterator TerminalsBegin() const;
	LVParseTree_TerminalIterator TerminalsEnd() const;

	//returns all the tags under a rule node
	//(equivalent to using a subtree iterator, and screening out all of the non-tags)
	LVParseTree_TagIterator TagsBegin() const;
	LVParseTree_TagIterator TagsEnd() const;


	LVParseTree_Node();
	LVParseTree_Node(const LVParseTree_Node& other);
	LVParseTree_Node& operator = (const LVParseTree_Node& other); 
	LVParseTree_Node(H_PARSE_TREE_NODE Node);//do not release handle with c api!!!
	~LVParseTree_Node();

private:
	 
	mutable H_PARSE_TREE_NODE N;

	friend class LVParseTree;
	friend class LVParseTree_Iterator;
	friend class LVParseTree_ChildrenIterator;
	friend class LVParseTree_TerminalIterator;
	friend class LVParseTree_TagIterator;
	friend class LVParseTree_ConceptIterator;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	class LVParseTree
//
//	LVParseTree is a container for the nodes that make up the parse tree (sentence diagram)
//  It also stores information about the grammar that built this tree, the score for the parse.
//  It doles out iterators over the trees nodes.  nodes, in turn, can dole out iterators over their subtrees.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////
class LVParseTree
{
public:
	//the LVParseTree is fully copyable.
	LVParseTree();
	LVParseTree(const LVParseTree& other);
	LVParseTree& operator=(const LVParseTree& other);
	~LVParseTree();

	bool operator==(const LVParseTree& other) const;
	bool operator!=(const LVParseTree& other) const {return !operator==(other);}

	///////////////////////////////////////////////////////////////////////////////////
	//
	// the format of the tags in the tree, as described in the tree's grammar.
	// for instance, "lumenvox/1.0" means the tree and its tags are intended to undergo 
	// semantic interpretation using the LumenVox interpreter.
	//
	///////////////////////////////////////////////////////////////////////////////////
	const char* TagFormat() const;

	///////////////////////////////////////////////////////////////////////////////////
	//
	//	for accessing the global tags defined in the matching grammar.
	//
	///////////////////////////////////////////////////////////////////////////////////
	int NumberOfTagsInHeader() const;
	const char* HeaderTag(int index) const;

	///////////////////////////////////////////////////////////////////////////////////
	//
	// For accessing properties about the grammar that was used to generate this tree:
	//
	// GrammarLabel() The name the grammar was given when it was entered into 
	//                the speechport for decoding.
	//
	// Mode()         "voice" or "dtmf", depending on whether this tree was 
	//                used to decode speech or dtmf.
	//
	// Language()     The language code for the matching grammar.  e.g. "en-US",
	//                "es-MX", "fr", "fr-CA"
	//
	///////////////////////////////////////////////////////////////////////////////////
	const char* GrammarLabel() const;
	const char* Mode() const;
	const char* Language() const;
	int         Score() const;

	typedef LVParseTree_Iterator Iterator;
	typedef LVParseTree_ChildrenIterator ChildrenIterator;
	typedef LVParseTree_TerminalIterator TerminalIterator;
	typedef LVParseTree_TagIterator TagIterator;
	typedef LVParseTree_ConceptIterator ConceptIterator;
	typedef LVParseTree_Node Node;

	///////////////////////////////////////////////////////////////////////////////////
	//
	// this pre-order iterator walks the entirety of the tree, beginning at the root, 
	// top to bottom, left to right.  See The LumenVox documentation pages for 
	// examples.
	//
	// calling Begin() is equivalent to calling Root().SubTreeBegin();
	// calling End() is equivalent to calling Root().SubTreeEnd();
	//
	///////////////////////////////////////////////////////////////////////////////////
	Iterator Begin() const;
	Iterator End() const;

	///////////////////////////////////////////////////////////////////////////////////
	//
	// Access the "concepts" of the parse tree. Concepts are a notion from the LumenVox
	// Concept/Phrase grammars.  In the language of parse trees, concepts are the
	// highest rules in the tree that have terminals as immediate children.
	//
	///////////////////////////////////////////////////////////////////////////////////
	ConceptIterator ConceptsBegin() const;
	ConceptIterator ConceptsEnd() const;
 
	///////////////////////////////////////////////////////////////////////////////////
	//
	// Returns the words that were detected in the order they were detected.
	// code:
	//    LVParseTree::TerminalIterator Itr = Tree.TerminalsBegin();
	//    LVParseTree::TerminalIterator End = Tree.TerminalsEnd();
	//    for (;Itr != End; ++Itr) {
	//        cout << Itr->Text() << " ";
	//    }
	//
	// is equivalent to:
	//    LVParseTree::Iterator Itr = Tree.Begin();
	//    LVParseTree::Iterator End = Tree.End();
	//    for (;Itr != End; ++Itr) if (Itr->IsTerminal()){
	//        cout << Itr->Text() << " ";
	//    }
	//
	///////////////////////////////////////////////////////////////////////////////////
	TerminalIterator TerminalsBegin() const;
	TerminalIterator TerminalsEnd() const;

	///////////////////////////////////////////////////////////////////////////////////
	//
	// Returns the tags in the tree in the order they were generated by the grammar and
	// utterance.
	// code:
	//    LVParseTree::TagIterator Itr = Tree.TagsBegin();
	//    LVParseTree::TagIterator End = Tree.TagsEnd();
	//    for (;Itr != End; ++Itr) {
	//        cout << Itr->Text() << " ";
	//    }
	//
	// is equivalent to:
	//    LVParseTree::Iterator Itr = Tree.Begin();
	//    LVParseTree::Iterator End = Tree.End();
	//    for (;Itr != End; ++Itr) if (Itr->IsTag()){
	//        cout << Itr->Text() << " ";
	//    }
	//
	///////////////////////////////////////////////////////////////////////////////////
	TagIterator TagsBegin() const;
	TagIterator TagsEnd() const;

	///////////////////////////////////////////////////////////////////////////////////
	//
	// One might ask, how do I access the ChildrenIterators?  
	// Answer:  What do you want the children of?  Trees don't have children; 
	//          only Tree Nodes have children.
	//          Call Tree.Root().ChildrenBegin()
	//          and  Tree.Root().ChildrenEnd()
	//
	///////////////////////////////////////////////////////////////////////////////////

	// root node of this tree.
	Node Root() const;


	H_PARSE_TREE Handle() const;
	H_PARSE_TREE GetH_SPT() const {return Handle();}//deprecated
	LVParseTree(H_PARSE_TREE Tree); //this object takes ownership of the memory for this handle.  
	                                //do not release it with the C API!!!
private:
	
	mutable H_PARSE_TREE T;
	Node R;

	friend class LVSpeechPort;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class LVParseTree_Iterator //also known as LVParseTree::Iterator
{
public:
	LVParseTree_Iterator();
	LVParseTree_Iterator(const LVParseTree::Iterator& other);
	~LVParseTree_Iterator();
	LVParseTree_Iterator&   operator =  (const LVParseTree::Iterator& other);

	LVParseTree_Iterator&   operator++ (void);
	LVParseTree_Iterator    operator++ (int);
	LVParseTree_Node const* operator-> (void) const;
	const LVParseTree_Node& operator*  (void) const;
	bool                     operator== (const LVParseTree::Iterator& other) const;
	bool                     operator!= (const LVParseTree::Iterator& other) const;
	LVParseTree_Iterator(H_PARSE_TREE_ITR Iterator);//takes control of handle. do not release with c api!!!
private:
	
	mutable H_PARSE_TREE_ITR I;
	LVParseTree_Node N; 

	friend class LVParseTree;
	friend class LVParseTree_Node;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class LVParseTree_TerminalIterator //also known as LVParseTree::TerminalIterator
{
public:
	LVParseTree_TerminalIterator();
	LVParseTree_TerminalIterator(const LVParseTree::TerminalIterator& other);
	~LVParseTree_TerminalIterator();
	LVParseTree_TerminalIterator& operator= (const LVParseTree::TerminalIterator& other);

	LVParseTree_TerminalIterator& operator++ (void);
	LVParseTree_TerminalIterator  operator++ (int);
	LVParseTree_Node const*       operator-> (void) const;
	const LVParseTree_Node&       operator*  (void) const;
	bool                           operator== (const LVParseTree::TerminalIterator& other) const;
	bool                           operator!= (const LVParseTree::TerminalIterator& other) const;
	LVParseTree_TerminalIterator(H_PARSE_TREE_TERMINAL_ITR Iterator);//do not release handle with c api!!!
private:
	
	mutable H_PARSE_TREE_TERMINAL_ITR I;
	LVParseTree::Node N;

	friend class LVParseTree;
	friend class LVParseTree_Node;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class LVParseTree_TagIterator //also known as LVParseTree::TagIterator
{
public:
	LVParseTree_TagIterator();
	LVParseTree_TagIterator(const LVParseTree::TagIterator& other);
	~LVParseTree_TagIterator();
	LVParseTree_TagIterator& operator= (const LVParseTree::TagIterator& other);

	LVParseTree_TagIterator& operator++ (void);
	LVParseTree_TagIterator  operator++ (int);
	LVParseTree_Node const*       operator-> (void) const;
	const LVParseTree::Node&       operator*  (void) const;
	bool                           operator== (const LVParseTree::TagIterator& other) const;
	bool                           operator!= (const LVParseTree::TagIterator& other) const;
	LVParseTree_TagIterator(H_PARSE_TREE_TAG_ITR Iterator);//do not release with c api!!!
private:
	
	mutable H_PARSE_TREE_TAG_ITR I;
	LVParseTree_Node N;

	friend class LVParseTree;
	friend class LVParseTree_Node;
};


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class LVParseTree_ChildrenIterator
{
public:
	LVParseTree_ChildrenIterator();
	LVParseTree_ChildrenIterator(const LVParseTree::ChildrenIterator& other);
	~LVParseTree_ChildrenIterator();

	LVParseTree::ChildrenIterator&   operator =  (const LVParseTree::ChildrenIterator& other);
	LVParseTree::ChildrenIterator&   operator ++ (void);
	LVParseTree::ChildrenIterator    operator ++ (int);
	LVParseTree::Node const*         operator -> (void) const;
	const LVParseTree::Node&         operator *  (void) const;
	bool                             operator == (const LVParseTree::ChildrenIterator& other) const;
	bool                             operator != (const LVParseTree::ChildrenIterator& other) const;
	LVParseTree_ChildrenIterator(H_PARSE_TREE_CHILDREN_ITR Iterator);//do not release handle with c api!!!
private:
	
	mutable H_PARSE_TREE_CHILDREN_ITR I;
	LVParseTree::Node N; 

	friend class LVParseTree;
	friend class LVParseTree_Node;
};


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class LVParseTree_ConceptIterator
{
public:
	LVParseTree_ConceptIterator();
	LVParseTree_ConceptIterator(const LVParseTree::ConceptIterator& other);
	~LVParseTree_ConceptIterator();
	LVParseTree_ConceptIterator&   operator =  (const LVParseTree_ConceptIterator& other);
	LVParseTree_ConceptIterator&   operator ++ (void);
	LVParseTree_ConceptIterator    operator ++ (int);
	LVParseTree_Node const*        operator -> (void) const;
	const LVParseTree_Node&        operator *  (void) const;
	bool                            operator == (const LVParseTree_ConceptIterator& other) const;
	bool                            operator != (const LVParseTree_ConceptIterator& other) const;
	LVParseTree_ConceptIterator(H_PARSE_TREE_CONCEPT_ITR Iterator);//do not release handle with c api!!!
private:
	mutable H_PARSE_TREE_CONCEPT_ITR I;
	LVParseTree_Node N; 

	friend class LVParseTree;
	friend class LVParseTree_Node;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Inline Functions
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

inline 
LVParseTree::Node LVParseTree::Root() const
{
	return R;
}

//======================================================================================================================

inline
LVParseTree::LVParseTree():T(NULL),R(NULL)
{
	T = LVParseTree_Create();
	R = Node(LVParseTree_GetRoot(T));
}

//======================================================================================================================

inline
LVParseTree::LVParseTree(const LVParseTree& other) : T(NULL),R(NULL)
{
	T = LVParseTree_CreateFromCopy(other.T);
	R = Node(LVParseTree_GetRoot(T));
}

//======================================================================================================================

inline
LVParseTree& LVParseTree::operator=(const LVParseTree& other)
{
	if(T == NULL)
		T = LVParseTree_CreateFromCopy(other.T);
	else
		LVParseTree_Copy(T,other.T);
	R = Node(LVParseTree_GetRoot(T));
	return *this;
}

//======================================================================================================================

inline
LVParseTree::~LVParseTree()
{
	LVParseTree_Release(T);
}
//======================================================================================================================
//takes control of Tree node
inline 
LVParseTree::LVParseTree(H_PARSE_TREE Tree)// : T(Tree)
{
	T = (Tree);
	R = Node(LVParseTree_GetRoot(T));
}

//======================================================================================================================

inline 
const char* LVParseTree::HeaderTag(int index) const
{
	return LVParseTree_GetHeaderTag(T,index);
}

//======================================================================================================================

inline 
const char* LVParseTree::TagFormat() const
{
	return LVParseTree_GetTagFormat(T);
}

//======================================================================================================================

inline
int LVParseTree::NumberOfTagsInHeader() const
{
	return LVParseTree_GetNumberOfTagsInHeader(T);
}

//======================================================================================================================

inline
const char* LVParseTree::GrammarLabel() const
{
	return LVParseTree_GetGrammarLabel(T);
}

//======================================================================================================================

inline
const char* LVParseTree::Language() const
{
	return LVParseTree_GetLanguage(T);
}

//======================================================================================================================

inline
int LVParseTree::Score() const
{
	return LVParseTree_GetScore(T);
}

//======================================================================================================================

inline
const char* LVParseTree::Mode() const
{
	return LVParseTree_GetMode(T);
}

//======================================================================================================================
inline
H_PARSE_TREE LVParseTree::Handle() const
{
	return T;
}

//======================================================================================================================

inline
bool LVParseTree::operator==(const LVParseTree& other) const
{
	return LVParseTree_AreEqual(T,other.T) != 0;
}

//======================================================================================================================

inline
LVParseTree::Iterator LVParseTree::Begin() const
{
	H_PARSE_TREE_ITR Itr = LVParseTree_CreateIteratorBegin(T);
	return Iterator(Itr);
}

//======================================================================================================================

inline
LVParseTree::Iterator LVParseTree::End() const
{
	H_PARSE_TREE_ITR Itr = LVParseTree_CreateIteratorEnd(T);
	return Iterator(Itr);
}

//======================================================================================================================

inline
LVParseTree::ConceptIterator LVParseTree::ConceptsBegin() const
{
	H_PARSE_TREE_CONCEPT_ITR Itr = LVParseTree_CreateConceptIteratorBegin(T);
	return ConceptIterator(Itr);
}

//======================================================================================================================

inline
LVParseTree::ConceptIterator LVParseTree::ConceptsEnd() const
{
	H_PARSE_TREE_CONCEPT_ITR Itr = LVParseTree_CreateConceptIteratorEnd(T);
	return ConceptIterator(Itr);
}

//======================================================================================================================

inline
LVParseTree::TerminalIterator LVParseTree::TerminalsBegin() const
{
	H_PARSE_TREE_TERMINAL_ITR Itr = LVParseTree_CreateTerminalIteratorBegin(T);
	return TerminalIterator(Itr);
}

//======================================================================================================================

inline
LVParseTree::TerminalIterator LVParseTree::TerminalsEnd() const
{
	H_PARSE_TREE_TERMINAL_ITR Itr = LVParseTree_CreateTerminalIteratorEnd(T);
	return TerminalIterator(Itr);
}

//======================================================================================================================

inline
LVParseTree::TagIterator LVParseTree::TagsBegin() const
{
	H_PARSE_TREE_TAG_ITR Itr = LVParseTree_CreateTagIteratorBegin(T);
	return TagIterator(Itr);
}

//======================================================================================================================

inline
LVParseTree::TagIterator LVParseTree::TagsEnd() const
{
	H_PARSE_TREE_TAG_ITR Itr = LVParseTree_CreateTagIteratorEnd(T);
	return TagIterator(Itr);
}

//======================================================================================================================

inline
LVParseTree_Iterator::LVParseTree_Iterator() : I(NULL)
{
	I = LVParseTree_Iterator_Create();
	N = LVParseTree::Node(LVParseTree_Iterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree_Iterator::LVParseTree_Iterator(const LVParseTree_Iterator& other) : I(NULL)
{
	I = LVParseTree_Iterator_CreateFromCopy(other.I);
	N = LVParseTree::Node(LVParseTree_Iterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree::Iterator& LVParseTree::Iterator::operator=(const LVParseTree::Iterator& other)
{
	if(I == NULL)
		I = LVParseTree_Iterator_CreateFromCopy(other.I);
	else
		LVParseTree_Iterator_Copy(I, other.I);
	N = LVParseTree::Node(LVParseTree_Iterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
LVParseTree_Iterator::~LVParseTree_Iterator()
{
	LVParseTree_Iterator_Release(I);
}

//======================================================================================================================

inline
LVParseTree::Iterator LVParseTree::Iterator::operator++(int)
{
	LVParseTree::Iterator Retval = *this;
	LVParseTree_Iterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_Iterator_GetNode(I));
	return Retval;
}

//======================================================================================================================

inline
LVParseTree::Iterator& LVParseTree::Iterator::operator++(void)
{
	LVParseTree_Iterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_Iterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
bool LVParseTree::Iterator::operator==(const LVParseTree::Iterator& other) const
{
	return LVParseTree_Iterator_AreEqual(I,other.I) == 0 ? false : true;
}

//======================================================================================================================

inline
bool LVParseTree::Iterator::operator!=(const LVParseTree::Iterator& other) const
{
	return LVParseTree_Iterator_AreEqual(I,other.I) == 0 ? true : false;
}
 
//======================================================================================================================

inline
const LVParseTree::Node* LVParseTree::Iterator::operator->(void) const
{
	return &N;
}

//======================================================================================================================

inline
const LVParseTree::Node& LVParseTree::Iterator::operator*(void) const
{
	return N;
}

//======================================================================================================================
//Object takes control of Iterator handle
inline
LVParseTree_Iterator::LVParseTree_Iterator(H_PARSE_TREE_ITR Itr) : I(Itr)
{
	N = I ? LVParseTree::Node(LVParseTree_Iterator_GetNode(I)) : NULL;
}

//======================================================================================================================





//======================================================================================================================

inline
LVParseTree_ChildrenIterator::LVParseTree_ChildrenIterator() : I(NULL)
{
	I = LVParseTree_ChildrenIterator_Create();
	N = LVParseTree::Node(LVParseTree_ChildrenIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree_ChildrenIterator::LVParseTree_ChildrenIterator(const LVParseTree_ChildrenIterator& other) : I(NULL)
{
	I = LVParseTree_ChildrenIterator_CreateFromCopy(other.I);
	N = LVParseTree::Node(LVParseTree_ChildrenIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree::ChildrenIterator& 
LVParseTree::ChildrenIterator::operator=(const LVParseTree::ChildrenIterator& other)
{
	if(I == NULL)
		I = LVParseTree_ChildrenIterator_CreateFromCopy(other.I);
	else
		LVParseTree_ChildrenIterator_Copy(I,other.I);
	N = LVParseTree::Node(LVParseTree_ChildrenIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
LVParseTree_ChildrenIterator::~LVParseTree_ChildrenIterator()
{
	LVParseTree_ChildrenIterator_Release(I);
}

//======================================================================================================================

inline
LVParseTree::ChildrenIterator LVParseTree::ChildrenIterator::operator++(int)
{
	LVParseTree::ChildrenIterator Retval = *this;
	LVParseTree_ChildrenIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_ChildrenIterator_GetNode(I));
	return Retval;
}

//======================================================================================================================

inline
LVParseTree::ChildrenIterator& LVParseTree::ChildrenIterator::operator++(void)
{
	LVParseTree_ChildrenIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_ChildrenIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
bool LVParseTree::ChildrenIterator::operator==(const LVParseTree::ChildrenIterator& other) const
{
	return LVParseTree_ChildrenIterator_AreEqual(I,other.I) == 0 ? false : true;
}

//======================================================================================================================

inline
bool LVParseTree::ChildrenIterator::operator!=(const LVParseTree::ChildrenIterator& other) const
{
	return LVParseTree_ChildrenIterator_AreEqual(I,other.I) == 0 ? true : false;
}
 
//======================================================================================================================

inline
const LVParseTree::Node* LVParseTree::ChildrenIterator::operator->(void) const
{
	return &N;
}

//======================================================================================================================

inline
const LVParseTree::Node& LVParseTree::ChildrenIterator::operator*(void) const
{
	return N;
}

//======================================================================================================================
//Object takes control of Iterator handle
inline
LVParseTree_ChildrenIterator::LVParseTree_ChildrenIterator(H_PARSE_TREE_CHILDREN_ITR Itr) : I(Itr)
{
	N = LVParseTree::Node(LVParseTree_ChildrenIterator_GetNode(I));
}

//======================================================================================================================





//======================================================================================================================

inline
LVParseTree_TerminalIterator::LVParseTree_TerminalIterator() : I(NULL)
{
	I = LVParseTree_TerminalIterator_Create();
	N = LVParseTree::Node(LVParseTree_TerminalIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree_TerminalIterator::LVParseTree_TerminalIterator(const LVParseTree_TerminalIterator& other) : I(NULL)
{
	I = LVParseTree_TerminalIterator_CreateFromCopy(other.I);
	N = LVParseTree::Node(LVParseTree_TerminalIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree::TerminalIterator& 
LVParseTree::TerminalIterator::operator=(const LVParseTree::TerminalIterator& other)
{
	if(I == NULL)
		I = LVParseTree_TerminalIterator_CreateFromCopy(other.I);
	else
		LVParseTree_TerminalIterator_Copy(I, other.I);
	N = LVParseTree::Node(LVParseTree_TerminalIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
LVParseTree_TerminalIterator::~LVParseTree_TerminalIterator()
{
	LVParseTree_TerminalIterator_Release(I);
}

//======================================================================================================================

inline
LVParseTree::TerminalIterator LVParseTree::TerminalIterator::operator++(int)
{
	LVParseTree::TerminalIterator Retval = *this;
	LVParseTree_TerminalIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_TerminalIterator_GetNode(I));
	return Retval;
}

//======================================================================================================================

inline
LVParseTree::TerminalIterator& LVParseTree::TerminalIterator::operator++(void)
{
	LVParseTree_TerminalIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_TerminalIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
bool LVParseTree::TerminalIterator::operator==(const LVParseTree::TerminalIterator& other) const
{
	return LVParseTree_TerminalIterator_AreEqual(I,other.I) == 0 ? false : true;
}

//======================================================================================================================

inline
bool LVParseTree::TerminalIterator::operator!=(const LVParseTree::TerminalIterator& other) const
{
	return LVParseTree_TerminalIterator_AreEqual(I,other.I) == 0 ? true : false;
}
 
//======================================================================================================================

inline
const LVParseTree::Node* LVParseTree::TerminalIterator::operator->(void) const
{
	return &N;
}

//======================================================================================================================

inline
const LVParseTree::Node& LVParseTree::TerminalIterator::operator*(void) const
{
	return N;
}

//======================================================================================================================
//Object takes control of Iterator handle
inline
LVParseTree_TerminalIterator::LVParseTree_TerminalIterator(H_PARSE_TREE_TERMINAL_ITR Itr) : I(Itr)
{
	N = LVParseTree::Node(LVParseTree_TerminalIterator_GetNode(I));
}

//======================================================================================================================





//======================================================================================================================

inline
LVParseTree_TagIterator::LVParseTree_TagIterator() : I(NULL)
{
	I = LVParseTree_TagIterator_Create();
	N = LVParseTree::Node(LVParseTree_TagIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree_TagIterator::LVParseTree_TagIterator(const LVParseTree_TagIterator& other) : I(NULL)
{
	I = LVParseTree_TagIterator_CreateFromCopy(other.I);
	N = LVParseTree::Node(LVParseTree_TagIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree::TagIterator& 
LVParseTree::TagIterator::operator=(const LVParseTree::TagIterator& other)
{
	if(I == NULL)
		I = LVParseTree_TagIterator_CreateFromCopy(other.I);
	else
		LVParseTree_TagIterator_Copy(I, other.I);
	N = LVParseTree::Node(LVParseTree_TagIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
LVParseTree_TagIterator::~LVParseTree_TagIterator()
{
	LVParseTree_TagIterator_Release(I);
}

//======================================================================================================================

inline
LVParseTree::TagIterator LVParseTree::TagIterator::operator++(int)
{
	LVParseTree::TagIterator Retval = *this;
	LVParseTree_TagIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_TagIterator_GetNode(I));
	return Retval;
}

//======================================================================================================================

inline
LVParseTree::TagIterator& LVParseTree::TagIterator::operator++(void)
{
	LVParseTree_TagIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_TagIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
bool LVParseTree::TagIterator::operator==(const LVParseTree::TagIterator& other) const
{
	return LVParseTree_TagIterator_AreEqual(I,other.I) == 0 ? false : true;
}

//======================================================================================================================

inline
bool LVParseTree::TagIterator::operator!=(const LVParseTree::TagIterator& other) const
{
	return LVParseTree_TagIterator_AreEqual(I,other.I) == 0 ? true : false;
}
 
//======================================================================================================================

inline
const LVParseTree::Node* LVParseTree::TagIterator::operator->(void) const
{
	return &N;
}

//======================================================================================================================

inline
const LVParseTree::Node& LVParseTree::TagIterator::operator*(void) const
{
	return N;
}

//======================================================================================================================
//Object takes control of Iterator handle
inline
LVParseTree_TagIterator::LVParseTree_TagIterator(H_PARSE_TREE_TAG_ITR Itr) : I(Itr)
{
	N = LVParseTree::Node(LVParseTree_TagIterator_GetNode(I));
}





//======================================================================================================================

inline
LVParseTree_ConceptIterator::LVParseTree_ConceptIterator() : I(NULL)
{
	I = LVParseTree_ConceptIterator_Create();
	N = LVParseTree::Node(LVParseTree_ConceptIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree_ConceptIterator::LVParseTree_ConceptIterator(const LVParseTree_ConceptIterator& other) : I(NULL)
{
	I = LVParseTree_ConceptIterator_CreateFromCopy(other.I);
	N = LVParseTree::Node(LVParseTree_ConceptIterator_GetNode(I));
}

//======================================================================================================================

inline
LVParseTree::ConceptIterator& 
LVParseTree::ConceptIterator::operator=(const LVParseTree::ConceptIterator& other)
{
	if(I == NULL)
		I = LVParseTree_ConceptIterator_CreateFromCopy(other.I);
	else
		LVParseTree_ConceptIterator_Copy(I, other.I);
	N = LVParseTree::Node(LVParseTree_ConceptIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
LVParseTree_ConceptIterator::~LVParseTree_ConceptIterator()
{
	LVParseTree_ConceptIterator_Release(I);
}

//======================================================================================================================

inline
LVParseTree::ConceptIterator LVParseTree::ConceptIterator::operator++(int)
{
	LVParseTree::ConceptIterator Retval = *this;
	LVParseTree_ConceptIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_ConceptIterator_GetNode(I));
	return Retval;
}

//======================================================================================================================

inline
LVParseTree::ConceptIterator& LVParseTree::ConceptIterator::operator++(void)
{
	LVParseTree_ConceptIterator_Advance(I);
	N = LVParseTree::Node(LVParseTree_ConceptIterator_GetNode(I));
	return *this;
}

//======================================================================================================================

inline
bool LVParseTree::ConceptIterator::operator==(const LVParseTree::ConceptIterator& other) const
{
	return LVParseTree_ConceptIterator_AreEqual(I,other.I) == 0 ? false : true;
}

//======================================================================================================================

inline
bool LVParseTree::ConceptIterator::operator!=(const LVParseTree::ConceptIterator& other) const
{
	return LVParseTree_ConceptIterator_AreEqual(I,other.I) == 0 ? true : false;
}
 
//======================================================================================================================

inline
const LVParseTree::Node* LVParseTree::ConceptIterator::operator->(void) const
{
	return &N;
}

//======================================================================================================================

inline
const LVParseTree::Node& LVParseTree::ConceptIterator::operator*(void) const
{
	return N;
}

//======================================================================================================================
//Object takes control of Iterator handle
inline
LVParseTree_ConceptIterator::LVParseTree_ConceptIterator(H_PARSE_TREE_CONCEPT_ITR Itr) : I(Itr)
{
	N = LVParseTree::Node(LVParseTree_ConceptIterator_GetNode(I));
}






//======================================================================================================================



inline
LVParseTree_Node::LVParseTree_Node() : N(NULL)
{}

//======================================================================================================================

inline
LVParseTree_Node::LVParseTree_Node(const LVParseTree_Node& other) : N(NULL)
{
	N = LVParseTree_Node_CreateCopy(other.N);
}

//======================================================================================================================

inline
LVParseTree::Node& LVParseTree::Node::operator=(const LVParseTree::Node& other)
{
	if(N)
		LVParseTree_Release_Node(N);
	N = LVParseTree_Node_CreateCopy(other.N);
	return *this;
}

//======================================================================================================================

inline
LVParseTree_Node::LVParseTree_Node(H_PARSE_TREE_NODE N) :N(N){}

//======================================================================================================================

inline
LVParseTree_Node::~LVParseTree_Node()
{
	LVParseTree_Release_Node(N);
}

//======================================================================================================================

inline
LVParseTree::ChildrenIterator LVParseTree::Node::ChildrenBegin() const
{
	H_PARSE_TREE_CHILDREN_ITR I = LVParseTree_Node_CreateChildrenIteratorBegin(N);
	return LVParseTree::ChildrenIterator(I);
}

//======================================================================================================================

inline
LVParseTree::ChildrenIterator LVParseTree::Node::ChildrenEnd() const
{
	H_PARSE_TREE_CHILDREN_ITR I = LVParseTree_Node_CreateChildrenIteratorEnd(N);
	return LVParseTree::ChildrenIterator(I);
}

//======================================================================================================================

inline
LVParseTree::Iterator LVParseTree::Node::SubTreeBegin() const
{
	H_PARSE_TREE_ITR I = LVParseTree_Node_CreateIteratorBegin(N);
	return LVParseTree::Iterator(I);
}

//======================================================================================================================

inline
LVParseTree::Iterator LVParseTree::Node::SubTreeEnd() const
{
	H_PARSE_TREE_ITR I = LVParseTree_Node_CreateIteratorEnd(N);
	return LVParseTree::Iterator(I);
}

//======================================================================================================================

inline
LVParseTree::TagIterator LVParseTree::Node::TagsBegin() const
{
	H_PARSE_TREE_TAG_ITR I = LVParseTree_Node_CreateTagIteratorBegin(N);
	return LVParseTree::TagIterator(I);
}

//======================================================================================================================

inline
LVParseTree::TagIterator LVParseTree::Node::TagsEnd() const
{
	H_PARSE_TREE_TAG_ITR I = LVParseTree_Node_CreateTagIteratorEnd(N);
	return LVParseTree::TagIterator(I);
}

//=====================================================================================================================

inline
LVParseTree::TerminalIterator LVParseTree::Node::TerminalsBegin() const
{
	H_PARSE_TREE_TERMINAL_ITR I = LVParseTree_Node_CreateTerminalIteratorBegin(N);
	return LVParseTree::TerminalIterator(I);
}

//======================================================================================================================

inline
LVParseTree::TerminalIterator LVParseTree::Node::TerminalsEnd() const
{
	H_PARSE_TREE_TERMINAL_ITR I = LVParseTree_Node_CreateTerminalIteratorEnd(N);
	return LVParseTree::TerminalIterator(I);
}

//=====================================================================================================================

inline
bool LVParseTree::Node::IsRule() const
{
	return LVParseTree_Node_IsRule(N) == 0 ? false : true;
}
 
//======================================================================================================================

inline
bool LVParseTree::Node::IsTag() const
{
	return LVParseTree_Node_IsTag(N) == 0 ? false : true;
}

//======================================================================================================================

inline
bool LVParseTree::Node::IsTerminal() const
{
	return LVParseTree_Node_IsTerminal(N) == 0 ? false : true;
}

//======================================================================================================================

inline
int LVParseTree::Node::Level() const
{
	return LVParseTree_Node_GetLevel(N);
}

//======================================================================================================================

inline
const char* LVParseTree::Node::RuleName() const
{
	return LVParseTree_Node_GetRuleName(N);
}

//======================================================================================================================

inline
int LVParseTree::Node::Score() const
{
	return LVParseTree_Node_GetScore(N);
}

//======================================================================================================================

inline
const char* LVParseTree::Node::Phonemes() const
{
	return LVParseTree_Node_GetPhonemes(N);
}

//======================================================================================================================

inline
int LVParseTree::Node::StartTime() const
{
	return LVParseTree_Node_GetStartTime(N);
}

//======================================================================================================================

inline
int LVParseTree::Node::BeginFrame() const
{
	return LVParseTree_Node_GetBeginFrame(N);
}

//======================================================================================================================

inline
int LVParseTree::Node::EndTime() const
{
	return LVParseTree_Node_GetEndTime(N);
}

//======================================================================================================================

inline
int LVParseTree::Node::EndFrame() const
{
	return LVParseTree_Node_GetEndFrame(N);
}

//======================================================================================================================

inline
const char* LVParseTree::Node::Text() const
{
	return LVParseTree_Node_GetText(N);
}

//======================================================================================================================

inline
const char* LVParseTree::Node::RawText() const
{
	return LVParseTree_Node_GetRawText(N);
}

//======================================================================================================================

inline
bool LVParseTree::Node::operator==(const LVParseTree::Node& other) const
{
	return LVParseTree_Node_AreEqual(N,other.N) != 0;
}

//======================================================================================================================


#endif//__cplusplus

#endif
