#include <LV_SRE.h>
#include <iostream>

#include <string>
#include <stdlib.h>
#include <stdio.h>

using namespace std;

void PrintUsage(const char *prog_name);

int main(int argc, char *argv[])
{
	// Handle command line inputs
	if(argc < 5 || argc > 6)
	{
		PrintUsage(argv[0]);

		return 1;
	}

	int snd_frmt = atoi(argv[3]);
	
	if(snd_frmt < 0 || snd_frmt > 4)
	{
		PrintUsage(argv[0]);
	
		return 1;
	}

	SOUND_FORMAT audio_format = static_cast<SOUND_FORMAT>(snd_frmt);

	string grammar_label = argv[1],
		grammar_file = argv[2],
		audio_file = argv[4];

	unsigned char* audio_buffer;

	int vc = 1;

	int retval, exit_code = 0;

	HPORT port;

	if(argc == 6)
	{
		LV_SRE_SetPropertyEx(NULL, PROP_EX_SRE_SERVERS, PROP_EX_VALUE_TYPE_STRING, argv[5], PROP_EX_TARGET_CLIENT, 0);
	}
	
	// Read Audio
	FILE *AUDIO = fopen(audio_file.c_str(), "rb");
	
	if (!AUDIO)
	{
		cerr << "Error opening audio file : " << audio_file << endl;
	
		return 2;
	}

	fseek(AUDIO, 0, SEEK_END);

	int flength = ftell(AUDIO);

	rewind(AUDIO);

	audio_buffer = new unsigned char[flength + 1];

	fread(audio_buffer, flength, 1, AUDIO);

	fclose(AUDIO);

	// 1. Open Port
	int OpenErrorCode = 0;
	
	port = LV_SRE_CreateClient(&OpenErrorCode, NULL, NULL, 0);
	
	if(port != NULL)
	{
		LV_SRE_SetPropertyEx(port, PROP_EX_SAVE_SOUND_FILES, PROP_EX_VALUE_TYPE_INT, (void *)1, PROP_EX_TARGET_PORT, 0);
	
		LV_SRE_SetPropertyEx(port, PROP_EX_MAX_NBEST_RETURNED, PROP_EX_VALUE_TYPE_INT, (void*)10, PROP_EX_TARGET_PORT, 0);

		LV_SRE_SetPropertyEx(port, PROP_EX_DECODE_TIMEOUT, PROP_EX_VALUE_TYPE_INT, (void*)100000, PROP_EX_TARGET_PORT, 0);

		// 2. Load Grammar
		//LV_SRE_IsGrammarLoaded(port, grammar_label.c_str());
		retval = LV_SRE_LoadGrammar(port, grammar_label.c_str(), grammar_file.c_str());
		if(retval == LV_SUCCESS)
		{
			cout << "Loaded Grammar!" << endl;

			// 3. Activate Grammar
			retval = LV_SRE_ActivateGrammar(port, grammar_label.c_str());
			if(retval == LV_SUCCESS)
			{
				// 4. Load Audio
				retval = LV_SRE_LoadVoiceChannel(port, vc, audio_buffer, flength, audio_format);
				if(retval == LV_SUCCESS)
				{
					cout << "Loaded Audio!" << endl;

					// 5. Do Decode
					retval = LV_SRE_Decode(port, vc, LV_ACTIVE_GRAMMAR_SET, LV_DECODE_BLOCK | LV_DECODE_SEMANTIC_INTERPRETATION);
					if(retval < 0)
					{
						cerr << "Decode failed. Error code : " << retval << endl;
						exit_code = 7;
					}
					else
						cout << "Decode returned!" << endl;
				}
				else
				{
					cerr << "Unable to load audio into voice channel(" << vc << "). Error code : " << retval << endl;
					exit_code = 6;
				}

				LV_SRE_UnloadGrammars(port);
			}
			else	// Activate Grammar Failed
			{
				cerr << "Unable to activate grammar " << grammar_label << ". Error code : " << retval << endl;
				exit_code = 5;
			}
		}
		else	// Load Grammar Failed
		{
			cerr << "Unable to load grammar " << grammar_file << ". Error code : " << retval << endl;
			exit_code = 4;
		}
	}
	else	// Open Port Failed
	{
		cerr << "Opening port failed with error code : " << OpenErrorCode << endl;
		exit_code = 3;
	}

	if(exit_code == 0)
	{
		// ==== Print Results ====
		// You may get multiple parses, and each parse may have multiple interpretations.
		int num_parses, p_idx, num_interps, i_idx;
		
		num_parses = LV_SRE_GetNumberOfNBestAlternatives(port,vc);

		if(num_parses > 0)
		{
			for(p_idx = 0; p_idx < num_parses; ++p_idx)
			{
				retval = LV_SRE_SwitchToNBestAlternative(port, vc, p_idx);

				if(retval != LV_SUCCESS)
				{
					cerr << "Unable to use alternative " << p_idx + 1 << " of " << num_parses << endl;
					exit_code = 8;
					break;
				}

				cout << "Alternative " << p_idx + 1 << ":" << endl;

				num_interps = LV_SRE_GetNumberOfInterpretations(port, vc);

				if(num_interps == 0)
				{
					cout << "  No Interpretations for this alternate: " << p_idx + 1 << endl;
				}
				else
				{
					for(i_idx = 0; i_idx < num_interps; ++i_idx)
					{
						H_SI si = LV_SRE_CreateInterpretation(port, vc, i_idx);

						const char * input_string = LVInterpretation_GetInputSentence(si);

						const char *gram_label = LVInterpretation_GetGrammarLabel(si);

						H_SI_DATA si_data = LVInterpretation_GetResultData(si);

						const char * interp_string = LVSemanticData_Print(si_data, SI_FORMAT_XML);

						int score = LVInterpretation_GetScore(si);

						cout << "  Interpretation " << i_idx + 1 << " of " << num_interps << ":" << endl;
						cout << "    Grammar Label        : " << (gram_label?gram_label:"") << endl;
						cout << "    Input Sentence       : " << (input_string?input_string:"") << endl;
						cout << "    Interpretation String: " << (interp_string?interp_string:"") << endl;
						cout << "    Interpretation Score : " << score << endl;

						LVSemanticData_Release(si_data);

						LVInterpretation_Release(si);
					}
				}

			}
		}
		else
		{
			cout << "No Decode results to report or there was a decode failure you might want to check the log files." << endl;
		}
	}	
	
	// 6. Close the port, do not forget to do this.
	if(exit_code != 3)
	{
		LV_SRE_DestroyClient(port);
	}

	LV_SRE_Shutdown();// flush out any pending background operations like writing callsre files

	delete [] audio_buffer;
	return exit_code;
}

void PrintUsage(const char *prog_name)
{
	cerr << "Usage: " << prog_name << " <grammar-label> <grammar-file> <sound-format> <audio-file> [server-ip]" << endl;
	cerr << "    where <sound-format> is one of:" << endl;
	cerr << "      0	for Unknown Format" << endl;
	cerr << "      1	for ULAW  8 kHz" << endl;
	cerr << "      2	for PCM   8 kHz" << endl;
	cerr << "      3	for PCM  16 kHz" << endl;
	cerr << "      4	for ALAW  8 kHz" << endl;
}

